//////////////////////////////////////////////////////////////////////////////
//
// 	Filename:	113-06210-001.c
//	Version:		0.01
//
//	Author:		Liu, Zemin  (jyetek@gmail.com)
//	Company:	JYE Tech
//	Web: 		www.jyetech.com
//
//-----------------------------------------------------------------------------
//
// 	Target: 		ATmega64
// 	Tool chain: 	WinAvr (GCC-AVR)
//
//-----------------------------------------------------------------------------
//	Application: 	DSO 062 (Learning Edition)
//
//-----------------------------------------------------------------------------
//	ATTENTION: 
//		1. The Clock Option (Bit 4 in High Byte) must be programmed before Low Byte
//			is programmed for high frequency crystal like 20MHz. Otherwise, it may
//			result in ISP failure because of not working oscillator.
//		2. Remember to program BOOTRST bit (Bit 0 of High Byte) for bootloader.
//
//	Notes:
//		1. Fuse configuration: 
//			Extended Byte: 	0b------11 (Non mega603/103 compatible mode)
//					Bit 1		- 1 = M64 mode, 0 = M103 mode
//					Bit 0		- 1 = WDT normal, 0 = WDT always ON
//			High Byte: 	0b11-00001 (CLKOP = 0 for crystal, EEPROM perserved)
//					Bit 7		- 1 = OCD disabled
//					Bit 6		- 1 = JTAG disabled
//					Bit 5		- 1 = SPI program disabled
//					bit 4		- Clock options. See datasheet
//					bit 3		- 1 = EEPROM not preserved, 0 = EEPROM preserved
//					bit [2:1]	- Boot size configration
//								Size				Boot loader Start (word address)
//							11 = 512 words		0x7E00
//							10 = 1024 words		0x7C00
//							01 = 2048 words		0x7800
//							00 = 4096 words		0x7000
//					bit 0		- 1 = Application Reset, 0 = Boot Loader Reset
//			Low Byte:	0b10101110 (Use external crystal osc [20MHz])
//					Bit 7		- BODLEVEL, 1 = 2.7V, 0 = 4.0V
//					Bit 6		- 1 = BOD disabled, 0 = BOD enabled
//					Bit [5:4]	- Start-up Time (see datasheet)
//					bit [3:0]	- Select clock source (See datasheet)
//
//		2. Add linker option "-Wl,--section-start=.noinit=0x801000" to makefile
//		    to set a fixed address for SampleBuf, which is defined used following
//		    method: 
//			"U8	SampleBuf[256] __attribute__ ((section (".noinit")));"
//
//-----------------------------------------------------------------------------
//	Revision History:
//		V0.01		2011.07.15
//		1.	Created based on 113-06208-101. 
//
///////////////////////////////////////////////////////////////////////////////

#include "Common.h"
#include "Board.h"
#include "Screen.h"
#include "FMeter.h"
#include	"Command.h"

PROGMEM	U8	CRLF[12] = {"\n\r"};
PROGMEM	U8	Title[20] = {"DSO 062 (LE)"};
PROGMEM	U8	Mfg[20] = {"By JYE Tech Ltd. "};
PROGMEM	U8	Website[20] = {"WWW.JYETECH.COM "};
PROGMEM	U8	Firmware[20] = {"FW: 06210-001"};

int	main (void)
{
 // -- Initialize ports
 Port_Init();

 // -- Initialize misc registers
 Misc_Init();

 Timer_Init();
 Comp_Init();
 ADC_Init();
 
  // -- Initialize Uart 0
 Uart0_Init();
 
 uputs_ROM(CRLF, Uart0);
 uputs_ROM(Title, Uart0);
 uputs_ROM(CRLF, Uart0);
 uputs_ROM(Mfg, Uart0);
 uputs_ROM(CRLF, Uart0);
 uputs_ROM(Website, Uart0);
 uputs_ROM(CRLF, Uart0);
 uputs_ROM(Firmware, Uart0);

 sei();
 
 LCD_Init();

 ScreenInit();
 
// Output two tone burst at PB1 and two LOW pulses at PE1 for troubleshooting
 BitSet8(DDRB, (1<<DDB1));
 SysMode = 200;
 BitClr8(PORTE, (1<<PE1));
 while(SysMode) {
 	BitSet8(PORTB, (1<<PB1));
	Delay(250);
 	BitClr8(PORTB, (1<<PB1));
	Delay(250);
	SysMode--;
 	}
 BitSet8(PORTE, (1<<PE1));
 Delay(60000);
 Delay(60000);
 Delay(60000);
 Delay(60000);
 Delay(60000);
 Delay(60000);
 SysMode = 200;
 BitClr8(PORTE, (1<<PE1));
 while(SysMode) {
 	BitSet8(PORTB, (1<<PB1));
	Delay(250);
 	BitClr8(PORTB, (1<<PB1));
	Delay(250);
	SysMode--;
 	}
 BitSet8(PORTE, (1<<PE1));
 
 BitClr8(DDRB, (1<<DDB1));

 // Application initialization
 ScopeInit();

 ShowWave();
 wDisplay();
 
 while(1) {
 	// Main loop
 	DetectSwitch();

 	switch(SysMode) {
		case ScopeMode:
			if(DSO_Setting.TimeBase > TB_50ms) {
				FlashMode();
 				}
 			else {
				RollingMode();
 				}

			// -- Key command processing
			KeyScan();
			if(KScanCode) {
				CmdProc();
				}		

			// Do not update indicators when in HOLD state
			if(!BitTest8(DSO_Status, (1<<Hold))) {
				UpdateCpl();
				UpdateVSen();
				}
			break;

		case FMeterMode:
			// Frequency meter mode
			MeasureFreq();
			break;

		default:
			break;
 		}
	
 	}

 return (0);

}

