//////////////////////////////////////////////////////////////////////////////
//
// 	Filename:	Board.h
//	Version:		
//
//	Author:		Liu, Zemin  (jyetek@gmail.com)
//	Company:	JYE Tech
//	Web: 		www.jyetech.com
//
//-----------------------------------------------------------------------------
//
// 	Target: 		ATmega64
// 	Tool chain: 	WinAvr (GCC-AVR)
//
//
//-----------------------------------------------------------------------------
//	Required files:
//
//-----------------------------------------------------------------------------
//	Notes:
//
//
//-----------------------------------------------------------------------------
//	Revision History:
//
///////////////////////////////////////////////////////////////////////////////
//
//

#ifndef	Board_h

#define	Board_h

#include	"Common.h"

// CPU frequency
//#define F_CPU      8000000L          	

// ===========================================
//	Uart parameters
//
#define	Uart0_Baud 		38400L
#define	Uart1_Baud 		38400L

#define	Uart0 			0
#define	Uart1			1

// LCD definitions
#define	LCD_Data_Port	PORTC
#define	LCD_Data_Dir	DDRC
#define	LCD_Data_In	PINC
#define	LCD_WR_Port	PORTG
#define	LCD_WR_Bit		PG3
#define	LCD_CS1_Port	PORTG
#define	LCD_CS1_Bit		PG0
#define	LCD_CS2_Port	PORTG
#define	LCD_CS2_Bit		PG1
#define	LCD_RES_Port	PORTG
#define	LCD_RES_Bit		PG2
#define	LCD_DI_Port		PORTG
#define	LCD_DI_Bit		PG4
#define	LCD_E_Port		PORTB
#define	LCD_E_Bit		PB0

#define	LCD_Delay		1

#define	LCD_I_Page0	0b10111000
#define	LCD_I_Y0		0b01000000
#define	LCD_I_On		0b00111111
#define	LCD_I_StartLn	0b11000000

#define	LCD_I_type		0
#define	LCD_D_type		1

#define	LCD_Block_Left	0
#define	LCD_Block_Right 1

// Key & switch definitions
#define	Key_Port_In		PIND
#define	Key_Bits		0x0F
#define	KeyScanWait	30		// Wait for 30ms if not key pressed
#define	KD_val			6		
#define	KH_val			4
#define	KR_val			3

#define	Key_Port_Out	PORTF
#define	Key_Col0		PF5
#define	Key_Col1		PF6
#define	Key_Col2		PF7
#define	Key_Drive_Col0	0xC0
#define	Key_Drive_Col1	0xA0
#define	Key_Drive_Col2	0x60
#define	Key_Drive_All	0x00

#define	SWDetOut_Port	PORTD
#define	SWDetOut_Bit	PD5
#define	SWDetIn_Port	PINB
#define	CplSwIn_Bit		PB1
#define	VSenSw1In_Bit	PB2
#define	VSenSw2In_Bit	PB3

// TrigOut definitions
#define	Trigout_Port		PORTB
#define	Trigout_Bit		PB4

extern	volatile	U16 	KScanCode;

// ====================================================
//	Macros
//

// ====================================================
//	Function Prototype Declarations
//
void	SYS_Init(void);
void	Misc_Init(void);
void Port_Init(void);
void Timer_Init (void);
void Comp_Init(void);
void ADC_Init (void);
void DelayTimer(U16 ms);
void Uart0_Init (void);

void	u0putc(U8 ch);
void	uPutc(U8 UartNum, U8 ch);
U8	u0getc(void);
void	uputs(U8 *s, U8 UartNum);
void	uputs_ROM(U8 *s, U8 UartNum);
U8	u0rcheck(void);

void	LCD_Init(void);
U8	LCD_Read(U8 type, U8 block);
U8	LCD_Write(U8 type, U8 date, U8 block);

void EE_Write(U8 *Addr, U8 Data);
U8 	EE_Read(U8 *Addr);
void	KeyScan(void);
void	DetectSwitch(void);


#endif	// Board_h


