//////////////////////////////////////////////////////////////////////////////
//
// 	Filename:	Command.c
//	Version:		
//
//	Author:		Liu, Zemin  (jyetek@gmail.com)
//	Company:	JYE Tech
//	Web: 		www.jyetech.com
//
//-----------------------------------------------------------------------------
//
// 	Target: 		ATmega64
// 	Tool chain: 	WinAvr (GCC-AVR)
//
//
//-----------------------------------------------------------------------------
//	Required files:
//
//-----------------------------------------------------------------------------
//	Notes:
//
//
//-----------------------------------------------------------------------------
//	Revision History:
//
///////////////////////////////////////////////////////////////////////////////
#include "Common.h"
#include "Board.h"
#include "Command.h"
#include "DSO.h"
#include	"Screen.h"
#include	"FMeter.h"

// ===========================================================
// 	File Scope Global variables
// ===========================================================
//

enum	Type_Focus 	Focus, PrevFocus;
enum	KeyStates  	KeyState;

U8	KeyCode;
U8	KeyParam;
		
// -- Key State Tables
#define	KST_Run_Size					12			
#define	KST_Hold_Size					10				
#define	KST_SaveWaveform_Size			5				
#define	KST_SaveWaveformEnd_Size		2				
#define	KST_DispWaveform_Size			5				

PROGMEM 	CmdEntry KST_Run[KST_Run_Size] = {
		{KVPos, 			KS_Run, 		KF_SelVPos	},
		{KHPos, 			KS_Run, 		KF_SelHPos	},
		{KTimeBase, 		KS_Run, 		KF_SelTimeBase},
		{KTrigMode,		KS_Run, 		KF_SelTrigMode},
		{KTrigEdge, 		KS_Run, 		KF_SelTrigEdge},
		{KTrigLvl, 		KS_Run, 		KF_SelTrigLevel},
		{KAdjUp, 		KS_Run, 		KF_AdjUp	},
		{KAdjDown, 		KS_Run, 		KF_AdjDown	},
		{KAdjUpH, 		KS_Run, 		KF_AdjUpH	},
		{KAdjDownH, 		KS_Run, 		KF_AdjDownH	},
		{KHold, 			KS_Hold, 	KF_Hold		},		
		{KVoid,	 		KS_Run, 		Dummy		}	
	};

PROGMEM 	CmdEntry KST_Hold[KST_Hold_Size] = {		
		{KVPos, 			KS_SaveWaveform, 	KF_ToSaveWaveform	},
		{KTrigMode,		KS_Hold,				KF_SelTrigMode},
		{KTrigEdge,		KS_DispWaveform,		KF_ToDispWaveform	},
		{KHPos, 			KS_Hold, 			KF_SelHPos	},
		{KAdjUp, 		KS_Hold, 			KF_AdjUp	},
		{KAdjDown, 		KS_Hold, 			KF_AdjDown	},
		{KAdjUpH, 		KS_Hold, 			KF_AdjUpH	},
		{KAdjDownH, 		KS_Hold, 			KF_AdjDownH	},
		{KHold, 			KS_Run, 				KF_ExitHold		},		
		{KVoid,	 		KS_Hold, 			Dummy		}	
	};

PROGMEM 	CmdEntry KST_SaveWaveform[KST_SaveWaveform_Size] = {		
		{KAdjUp, 		KS_SaveWaveform, 		KF_AdjUp	},
		{KAdjDown, 		KS_SaveWaveform, 		KF_AdjDown	},
		{KHold, 			KS_SaveWaveformEnd, 	KF_DoSaveWaveform		},		
		{KVPos,			KS_Hold,					KF_ExitSaveWaveform	},
		{KVoid,	 		KS_SaveWaveform, 		Dummy		}	
	};

PROGMEM 	CmdEntry KST_SaveWaveformEnd[KST_SaveWaveformEnd_Size] = {		
		{KHold, 			KS_Hold,					KF_ExitSaveWaveform	},		
		{KVoid,	 		KS_SaveWaveformEnd, 	Dummy		}	
	};

PROGMEM 	CmdEntry KST_DispWaveform[KST_DispWaveform_Size] = {		
		{KAdjUp, 		KS_DispWaveform, 	KF_AdjUp	},
		{KAdjDown, 		KS_DispWaveform, 	KF_AdjDown	},
		{KHold, 			KS_Hold, 			KF_DoDispWaveform		},		
		{KTrigEdge,		KS_Hold,				KF_ExitDispWaveform	},
		{KVoid,	 		KS_DispWaveform, 	Dummy		}	
	};

// -- Key State Index
PROGMEM	CmdEntry	*KIndex[KS_Last] = {
	(CmdEntry *)KST_Run,
	(CmdEntry *)KST_Hold,
	(CmdEntry *)KST_SaveWaveform,
	(CmdEntry *)KST_SaveWaveformEnd,
	(CmdEntry *)KST_DispWaveform,
	};

// -- Adjustment control table
// -- Adjustments available for Run State
PROGMEM	void (*RunState[FC_Last])(U8) = {
		Dummy1,		// Dummy entry
		AdjTimeBase,
		AdjHPos,
		AdjVPos,
		AdjTrigMode,
		AdjTrigEdge,
		AdjTrigLvl,
		};

// Key position
// Bit	8		7		6		5		4		3		2		1		0
// 		SW6		SW12	SW10	SW8		SW5		SW11	SW9		SW7		SW4
//		OK		VPOS	HPOS	S/DIV	+		MODE	SLOPE	LEVEL	-
PROGMEM 	KeyMap KMap[13] = {
	{0x01FE, KAdjDown	},
	{0x01FD, KTrigLvl		},
	{0x01FB, KTrigEdge	},
	{0x01F7, KTrigMode	},
	{0x01EF, KAdjUp		},
	{0x01DF, KTimeBase	},
	{0x01BF, KHPos		},
	{0x017F, KVPos		},
	{0x00FF, KHold		},
	{0x81EF, KAdjUpH		},
	{0x81FE, KAdjDownH	},
	{0x80FF, KHoldH		},
	{0, 		0			}	// End of table
	};

// ==========================================================
//	Function Definitions
// ==========================================================
//
// Map key scan code to key code
U8	KeyConvert(U16 KScancode, KeyMap *keymap)
{
 U16 tmp0;
 
 // -- Looking for match command code
 while((tmp0 = pgm_read_word((U8 *)keymap)) != 0) {
	if(tmp0 == KScancode) {
		// -- Match found
		return (pgm_read_byte((U8 *)keymap + 2));
		}
	else {
		// -- Proceed to next entry
		keymap = (KeyMap *)((U8 *)keymap+ sizeof(KeyMap));
		}
 	}

 return 0;
}


void	CmdProc(void)
{
 if(KScanCode != 0) {
 	// Convert to key code
 	KeyCode = KeyConvert(KScanCode, KMap);
	// -- Analyses key and execute associated key function
	(*(void (*)() )CmdAnalysis( 
		KeyCode, 
		(CmdEntry *)pgm_read_word(&KIndex[KeyState]),
		(U8 *)&KeyState))();
	KeyCode = 0;
	KScanCode = 0;
	}
}

// ----------------------------------------------------------------------------
// Resolve a command function with given 'CmdCode' and 'CmdTab'. Update 'PState' 
//	with NextState and return the pointer to the resolved command function
// ----------------------------------------------------------------------------
//
void	*CmdAnalysis(U8 Ccode, CmdEntry *CmdTab, U8 *PState)
{
 U8	tmp0;
 
 // -- Looking for match command code
 while((tmp0 = pgm_read_byte((U8 *)CmdTab)) != 0) {
	if(tmp0 == Ccode) {
		// -- Match found
		*PState = pgm_read_byte((U8 *)CmdTab + 1);
		return ((void *)pgm_read_word((U8 *)CmdTab + 2));
		}
	else {
		// -- Proceed to next entry
		CmdTab = (CmdEntry *)((U8 *)CmdTab + sizeof(CmdEntry));
		}
 	}

 return Dummy;
}


void CmdInit(void)
{
 KeyState = KS_Run;
 KeyCode = 0;

 Focus = FC_TBRate;
 PrevFocus = FC_TBRate;

}

void	KF_SelVPos(void)
{
 Focus = FC_VPos;
 UpdateFocus();
}

void	KF_SelHPos(void)
{
 Focus = FC_HPos;
 UpdateFocus();
}

void	KF_SelTimeBase(void)
{
 Focus = FC_TBRate;
 UpdateFocus();
 
}

void	KF_SelTrigMode(void)
{
 Focus = FC_TrigMode;
 UpdateFocus();
}

void	KF_SelTrigEdge(void)
{
 Focus = FC_TrigEdge;
 UpdateFocus();
 KF_AdjUp();
}

void	KF_SelTrigLevel(void)
{
 if(Focus == FC_Triglvl) {
 	KF_SelTrigSrc();
 	}
 Focus = FC_Triglvl;
 UpdateFocus();
}

void	KF_SelTrigSrc(void)
{
 // Change trig source (trig output) setting
 DSO_Setting.TrigSrc++;
 if(DSO_Setting.TrigSrc == TS_TrigOut + 1) {
 	DSO_Setting.TrigSrc = TS_Internal;
 	}

 if(DSO_Setting.TrigSrc == TS_TrigOut) {
 	// If TrigOutput is selected set the Test Signal pin as normal port operation mode
	BitClr8(TCCR0, (1<<COM00));
 	}
 else {
 	// Make the pin controlled by Timer0
	BitSet8(TCCR0, (1<<COM00));
 	}
 
 EE_Write((U8*)&DSO_Setting_EE.TrigSrc, DSO_Setting.TrigSrc);
 UpdateTrigSrc();
 UpdateTLvl();

}

void	KF_AdjUp(void)
{
 switch(KeyState) {
 	case KS_Run:
		KeyParam = 1;
		// uputs((U8 *)"AdjUp", 1);
 		ParamAdj(UP);
		break;
		
	case KS_Hold:
		if((Focus == FC_HPos)||(Focus == FC_TrigMode)) {
			KeyParam = 1;
 			ParamAdj(UP);
			}
		break;
		
	case KS_SaveWaveform:
		WFmemIndex++;
		if(WFmemIndex == MaxWFmem) {
			WFmemIndex = 0;
			}
		KF_ToSaveWaveform();
		break;

	case KS_DispWaveform:
		WFmemIndex++;
		if(WFmemIndex == MaxWFmem) {
			WFmemIndex = 0;
			}
		KF_ToDispWaveform();
		break;
		
	default:
		break;
		
 	}
}


void	KF_AdjDown(void)
{
 switch(KeyState) {
 	case KS_Run:
		KeyParam = 1;
 		ParamAdj(DOWN);
		break;
		
	case KS_Hold:
		if((Focus == FC_HPos)||(Focus == FC_TrigMode)) {
			KeyParam = 1;
 			ParamAdj(DOWN);
			}
		break;
		
	case KS_SaveWaveform:
		WFmemIndex--;
		if(WFmemIndex == 0xFF) {
			WFmemIndex = MaxWFmem - 1;
			}
		KF_ToSaveWaveform();
		break;
		
	case KS_DispWaveform:
		WFmemIndex--;
		if(WFmemIndex == 0xFF) {
			WFmemIndex = MaxWFmem - 1;
			}
		KF_ToDispWaveform();
		break;
		
	default:
		break;
		
 	}
}

void	KF_AdjUpH(void)
{
 switch(KeyState) {
 	case KS_Run:
		KeyParam = 5;
 		ParamAdj(UP);
		break;
		
	case KS_Hold:
		if(Focus == FC_HPos) {
			KeyParam = 5;
 			ParamAdj(UP);
			}
		break;
		
	case KS_SaveWaveform:
	case KS_DispWaveform:

	default:
		break;
 	}

}


void	KF_AdjDownH(void)
{
 switch(KeyState) {
 	case KS_Run:
		KeyParam = 5;
 		ParamAdj(DOWN);
		break;
		
	case KS_Hold:
		if(Focus == FC_HPos) {
			KeyParam = 5;
 			ParamAdj(DOWN);
			}
		break;
		
	case KS_SaveWaveform:
	case KS_DispWaveform:
		
	default:
		break;
 	}
}

void	KF_Hold(void)
{
 BitXor8(DSO_Status, (1<<Hold));
 UpdateHold();
 }

void	KF_ExitHold(void)
{
 BitXor8(DSO_Status, (1<<Hold));

 DSO_Setting.TimeBase = EE_Read((U8 *)&DSO_Setting_EE.TimeBase);
 DSO_Setting.HPos = EE_Read((U8 *)&DSO_Setting_EE.HPos);
 DSO_Setting.VPos = EE_Read((U8 *)&DSO_Setting_EE.VPos);
 DSO_Setting.TrigMode = EE_Read((U8 *)&DSO_Setting_EE.TrigMode);
 DSO_Setting.TrigEdge = EE_Read((U8 *)&DSO_Setting_EE.TrigEdge);
 DSO_Setting.TrigLvl = EE_Read((U8 *)&DSO_Setting_EE.TrigLvl);
 DSO_Setting.TrigSrc = EE_Read((U8 *)&DSO_Setting_EE.TrigSrc);
 DSO_Setting.TrigLvl_e = EE_Read((U8 *)&DSO_Setting_EE.TrigLvl_e);

 ScopePanel();
 
}

void KF_ToSaveWaveform(void)
{
 ClearDBuf(AscBuf);

 LcdPuts(5, 10, (U8 *)"Save WF to Buf", AscBuf);
 PutC5x7(93, 10, (U8)((WFmemIndex + 1) | 0x30), AscBuf);
 LcdPuts(5, 30, (U8 *)"[+][-] -> Sel buf", AscBuf);
 LcdPuts(5, 40, (U8 *)"[OK] -> Save", AscBuf);
 LcdPuts(5, 50, (U8 *)"[V.POS] -> Exit", AscBuf);
 
 Display(AscBuf); 

}

void KF_DoSaveWaveform(void)
{
 U16 tmp0, tmp1;
 U8  *ptmp;

 ClearDBuf(AscBuf);
 LcdPuts(5, 10, (U8 *)"Saving waveform...", AscBuf);
 Display(AscBuf); 

 // Current DSO settings
 ptmp = (U8 *)&DSO_Setting;
 
 // Waveform memory start address
 tmp1 = (U16)WFmem + WFmemIndex * WFmemSize;

 // Save current DSO settings
 tmp0 = 0;
 while(tmp0 < sizeof(DSO)) {
	EE_Write((U8*)tmp1, *ptmp);
	tmp1++;
	ptmp++;
	tmp0++;
 	}

 ptmp = (U8 *)(CaptureStart + 256);
 tmp1 = (U16)WFmem + WFmemIndex * WFmemSize + 32;
 // Save current waveform
 tmp0 = 0;
 while(tmp0 < WBsize) {
	EE_Write((U8*)tmp1, *ptmp);
	tmp1++;
	ptmp++;
	if(ptmp >= SampleBuf + 512) {
		ptmp -= 256;
		}
	tmp0++;
 	}

 // Feedback result
 ClearDBuf(AscBuf);
 LcdPuts(10, 10, (U8 *)"Save waveform done", AscBuf);

 LcdPuts(10, 40, (U8 *)"[OK] to return", AscBuf);
 Display(AscBuf); 

}

void KF_ExitSaveWaveform(void)
{
 ClearDBuf(DispBuf);
 ScopePanel();
 wDisplay();
 }

void KF_ToDispWaveform(void)
{
 ClearDBuf(AscBuf);

 LcdPuts(5, 10, (U8 *)"Display Buf", AscBuf);
 PutC5x7(74, 10, (U8)((WFmemIndex + 1) | 0x30), AscBuf);
 LcdPuts(5, 30, (U8 *)"[+][-] -> Sel buf", AscBuf);
 LcdPuts(5, 40, (U8 *)"[OK] -> Display", AscBuf);
 LcdPuts(5, 50, (U8 *)"[SLOPE] -> Exit", AscBuf);
 
 Display(AscBuf); 


}

void KF_DoDispWaveform(void)
{
 U16 tmp0, tmp1;
 U8  *ptmp;

 // Current DSO settings
 ptmp = (U8 *)&DSO_Setting;
 
 // Waveform memory start address
 tmp1 = (U16)WFmem + WFmemIndex * WFmemSize;

 // Load current DSO settings
 tmp0 = 0;
 while(tmp0 < sizeof(DSO)) {
	*ptmp = EE_Read((U8*)tmp1);
	tmp1++;
	ptmp++;
	tmp0++;
 	}

 ptmp = (U8 *)(SampleBuf + 256);
 tmp1 = (U16)WFmem + WFmemIndex * WFmemSize + 32;
 // Load current waveform
 tmp0 = 0;
 while(tmp0 < WBsize) {
	*ptmp = EE_Read((U8*)tmp1);
	tmp1++;
	ptmp++;
	tmp0++;
 	}

 CaptureStart = SampleBuf;
 ClearDBuf(DispBuf);
 ScopePanel();
 wClr();
 ShowWave();
 wDisplay();
 
}

void KF_ExitDispWaveform(void)
{
 ClearDBuf(DispBuf);
 ScopePanel();
 wDisplay();

}

void	UpdateFocus(void)
{ 
 // -- Clear highlight at old position
 Hightlight(PrevFocus);

 // -- Set highlight at new position
 Hightlight(Focus);

 // -- Save current position
 PrevFocus = Focus;

 //Update all symbol display
 PutBox(105, 23, 23, 39);
 
}

void	ParamAdj(U8 Dir)
{

 (*(void (*)())pgm_read_word(&RunState[Focus]))(Dir);
 
}


void Dummy(void)
{

}

void Dummy1(U8 Dir)
{

}

