//////////////////////////////////////////////////////////////////////////////
//
// 	Filename:	DSO.c
//	Version:		
//
//	Author:		Liu, Zemin  (jyetek@gmail.com)
//	Company:	JYE Tech
//	Web: 		www.jyetech.com
//
//-----------------------------------------------------------------------------
//
// 	Target: 		ATmega64
// 	Tool chain: 	WinAvr (GCC-AVR)
//
//
//-----------------------------------------------------------------------------
//	Required files:
//
//-----------------------------------------------------------------------------
//	Notes:
//
//
//-----------------------------------------------------------------------------
//	Revision History:
//
///////////////////////////////////////////////////////////////////////////////
//
//-----------------------------------------------------------------------------
// 	Includes
//-----------------------------------------------------------------------------

#include	"Common.h"
#include 	"DSO.h"
#include	"Screen.h"
#include	"Board.h"
#include	"Command.h"

// ===========================================================
// 	File Scope Global variables
// ===========================================================
//
U8	*CurrentSample;		// Address for saving current sample
U8	*CaptureStart;		// Starting point of current capture

U8	PrevSample;
U8	PrevVPos;
volatile	U8	PrevTLvl;
U8	PrevHPos;

volatile	U8	DSO_Status;
volatile	U8	SampleCount;
volatile	U8	TempSample;
volatile	U8	wUpdatePacer;
volatile	U16	TrigAddress;
volatile	U16	SamplePacer;
volatile	U16	TrigPos;		
volatile	U16	TrigPosC;	// This variable holds the rest buffer size after a trig

SystemMode	SysMode;	

U8	T0Scaler0, T0Scaler1;

U8	SampleBuf[512] __attribute__ ((section (".noinit")));

U8	Signature_EE1 __attribute__ ((section (".eeprom"))) = {EE_signature1};
U8	Signature_EE2 __attribute__ ((section (".eeprom"))) = {EE_signature2};

DSO	DSO_Setting_EE __attribute__ ((section (".eeprom"))) = {
	TB_1ms, 
	2,		
	45, 
	TM_Auto,
	TE_Falling,
	30,
	0,
	32
	}; 

// Waveform memory address (EEPROM)
volatile	U16	WFmem;

U8	WFmemIndex;

U8	WaveBuf[WLen * (Vmax/8)];	

// ================ Scope Operation Settings ================

DSO		DSO_Setting;
PROGMEM	DSO DSO_Setting_ROM	= {
		TB_1ms, 
		78,		
		45, 
		TM_Auto,
		TE_Falling,
		30,
		0,
		32
		};

PROGMEM 	FocusBox FBox[7] = {
		{0, 0, 23},		// dummy
		{105, 23, 23},		// Timebase
		{117, 32, 11},		// H position
		{105, 32, 11},		// V position
		{105, 43, 17},		// Trig mode
		{105, 52, 11},		// Trig edge
		{117, 52, 11}		// Trig level
	};

PROGMEM 	U8 TBString[TB_max][6] = {
	"dum",				// TB_min,
	" 10M",				// TB_10m,
	"  5M",				// TB_5m,
	"  2M",				// TB_2m,
	"  1M",				// TB_1m,
	" 50s",				//	TB_50s,
	" 20s",				//	TB_20s,
	" 10s",				//	TB_10s,
	"  5s",				//	TB_5s,
	"  2s",				//	TB_2s,
	"  1s",				//	TB_1s,
	"0.5s",				//	TB_05s,
	"0.2s",				//	TB_02s,
	"0.1s",				//	TB_01s,
	"50ms",				//	TB_50ms,
	"20ms",				//	TB_20ms,
	"10ms",				//	TB_10ms,
	" 5ms",				//	TB_5ms,
	" 2ms",				//	TB_2ms,
	" 1ms",				//	TB_1ms,
	".5ms",				//	TB_05ms,
	".2ms",				//	TB_02ms,
	".1ms"				//	TB_01ms,
						//	TB_max
	};

// Locations (x, y) of each adjustable item
PROGMEM 	U8 IndicatorLoc[(FC_Last + 1) * 2] = {
	0,	0,	// dummy
	106,	24,	// Timebase
	118,	33,	// HPos
	106,	33,	// VPos
	106,	44,	// Trig Mode
	106,	53,	// Trig edge
	118,	53,	// Trig level
	106, 15	// V Sensitivity
	};

PROGMEM 	U8 TMString[TM_Last][4] = {
	"dum",			// dummy
	"AUT",			// AUT
	"NOR",			// NOR
	"SIG"			// SIG
	};

PROGMEM 	U8 VSString[VS_Last][5] = {
	"dum",
	"0.1V",
	"0.2V",
	"0.5V",
	"  1V",
	"  2V",
	"  5V"
	};

PROGMEM 	U16	TBconstants[TB_max] = {
	TB_min,
	// -- SamplePacer constants used in RollingMode.
	60000,		//	TB_10m,		//		60s
	30000,		//	TB_5m,		//		30s
	12000,		//	TB_2m,		//		12s
	6000,		//	TB_1m,		//		6s
	5000,		//	TB_50s,		//		5s
	2000,		//	TB_20s,		// 		2s
	1000,		//	TB_10s,		// 1Hz	1s
	500,			//	TB_5s,		// 2Hz	500ms
	200,			//	TB_2s,		// 5Hz	200ms
	100,			//	TB_1s,		// 10Hz	100ms
	50,			//	TB_05s,		// 20Hz	50ms
	20,			//	TB_02s,		// 50Hz	20ms
	10,			//	TB_01s,		// 100Hz	10ms
	5,			//	TB_50ms,	// 200Hz	5ms
	
	// -- Constants for ADC clock in FlashMode
	40000,		//	TB_20ms,	// 500Hz	2ms
	20000,		//	TB_10ms,	// 1K	1ms
	10000,		//	TB_5ms,		// 2K	500us
	4000,		//	TB_2ms,		// 5K	200us
	2000,		//	TB_1ms,		// 10K	100us
	1000,		//	TB_05ms,	// 20K	50us
	400,			// 	TB_02ms,	// 50K	20us (400 cycles)
	200			// 	TB_01ms,	// 100K	10us (200 cycles)
//	200			// 	TB_max
	};

// Timing constants for display update in RollingMode
PROGMEM 	U8	UpdateConstants[TB_max] = {
	1,			// dummy
	1,			//	TB_10m,		//		60s
	1,			//	TB_5m,		//		30s
	1,			//	TB_2m,		//		12s
	1,			//	TB_1m,		//		6s
	1,			//	TB_50s,		//		5s
	1,			//	TB_20s,		// 		2s
	1,			//	TB_10s,		// 1Hz	1s
	1,			//	TB_5s,		// 2Hz	500ms
	1,			//	TB_2s,		// 5Hz	200ms
	2,			//	TB_1s,		// 10Hz	100ms
	4,			//	TB_05s,		// 20Hz	50ms
	10,			//	TB_02s,		// 50Hz	20ms
	20,			//	TB_01s,		// 100Hz	10ms
	40			//	TB_50ms,	// 200Hz	5ms
	};
	
// ===========================================================
// 	Function Definitions
// ===========================================================
//
// ----------------------------------------------------------------------------
// Display wave
//	Process data in WaveBuf and display it to screen
// ----------------------------------------------------------------------------
//
void	ShowWave(void)
{
 U8 tmp0, tmp1, tmp2, tmp3;
 U8 *ptmp1, *ptmp2;

 // Save current sample buffer if not in HOLD
 if(!BitTest8(DSO_Status, (1<<Hold))) {
 	ptmp1 = SampleBuf;
 	ptmp2 = ptmp1 + 256;
 	*ptmp2++ = *ptmp1++;
	 tmp0 = 255;
 	while(tmp0) {
 		*ptmp2++ = *ptmp1++;
		tmp0--;
 		}
 	}

 if(BitTest8(DSO_Status, (1<<SingleHold))) {
 	// Just finish one single mode captrue
 	BitSet8(DSO_Status, (1<<Hold));
	BitClr8(DSO_Status, (1<<SingleHold));
	KeyState = KS_Hold;
 	UpdateHold();
 	}

 ptmp1 = (U8 *)(CaptureStart + DSO_Setting.HPos);
 if(ptmp1 > (SampleBuf + WBsize)) {
	ptmp1 = (U8*)((U16)ptmp1 - WBsize);
 	}
 ptmp1 += 256;

 // -- Wave update 
 wClr();
 
 tmp0 = 0;
 while(tmp0 < 100) {
 	// Get a sample
	tmp1 = *ptmp1 >> 2;
	if(tmp1 > Lmax) {
		tmp1 = Lmax;
		}
	if(tmp1 < Lmin) {
		tmp1 = Lmin;
		}

	if(tmp0 == 0) {
		// -- Make previous point the same as current 
		PrevSample = tmp1;
		}

	tmp2 = Min(tmp1, PrevSample);
	tmp3 = Max(tmp1, PrevSample);
		
	PrevSample = tmp1;

	// -- Draw the vertival line segment between two points
	wVLine(tmp0, Lmax - tmp3, tmp3 - tmp2 + 1);
	
	ptmp1 += 1;
	if((U16)ptmp1 >= ((U16)SampleBuf + 256 + WBsize)) {
		ptmp1 = SampleBuf + 256;
		}
	tmp0++;
	}

}

// ----------------------------------------------------------------------------
//	Scope Initialization
// ----------------------------------------------------------------------------
//
void	ScopeInit(void)
{
 U8	tmp,  tmp1;
 	
 // Clear wave buffer
 wClr();
 
 CaptureStart = SampleBuf;
 while(CaptureStart!= SampleBuf + 255) {
 	*CaptureStart++ = 0x00;
 	}
 
 CaptureStart = SampleBuf;

 TrigPos = 64;
 TrigPosC = WBsize - TrigPos;
 WFmemIndex = 0;
 WFmem = 0x0100;

 // Looking for EEPROM signature
 tmp = EE_Read((U8 *)&Signature_EE1);
 tmp1 = EE_Read((U8 *)&Signature_EE2);

 if((tmp == EE_signature1) && (tmp1 == EE_signature2)) {
 	// Load setting from EEPROM
 	DSO_Setting.TimeBase = EE_Read((U8 *)&DSO_Setting_EE.TimeBase);
 	DSO_Setting.HPos = EE_Read((U8 *)&DSO_Setting_EE.HPos);
 	DSO_Setting.VPos = EE_Read((U8 *)&DSO_Setting_EE.VPos);
	DSO_Setting.TrigMode = EE_Read((U8 *)&DSO_Setting_EE.TrigMode);
	DSO_Setting.TrigEdge = EE_Read((U8 *)&DSO_Setting_EE.TrigEdge);
 	DSO_Setting.TrigLvl = EE_Read((U8 *)&DSO_Setting_EE.TrigLvl);
 	DSO_Setting.TrigSrc = EE_Read((U8 *)&DSO_Setting_EE.TrigSrc);
 	DSO_Setting.TrigLvl_e = EE_Read((U8 *)&DSO_Setting_EE.TrigLvl_e);	
 	}
 else {
 	// Load setting from Flash
 	DSO_Setting.TimeBase = pgm_read_byte((U8 *)&DSO_Setting_ROM.TimeBase);
 	DSO_Setting.HPos = pgm_read_byte((U8 *)&DSO_Setting_ROM.HPos);
 	DSO_Setting.VPos = pgm_read_byte((U8 *)&DSO_Setting_ROM.VPos);
	DSO_Setting.TrigMode = pgm_read_byte((U8 *)&DSO_Setting_ROM.TrigMode);
	DSO_Setting.TrigEdge = pgm_read_byte((U8 *)&DSO_Setting_ROM.TrigEdge);
 	DSO_Setting.TrigLvl = pgm_read_byte((U8 *)&DSO_Setting_ROM.TrigLvl);
 	DSO_Setting.TrigSrc = pgm_read_byte((U8 *)&DSO_Setting_ROM.TrigSrc);
 	DSO_Setting.TrigLvl_e = pgm_read_byte((U8 *)&DSO_Setting_ROM.TrigLvl_e);

	// Setup EEPROM
 	EE_Write((U8*)&DSO_Setting_EE.TimeBase, DSO_Setting.TimeBase);
 	EE_Write((U8*)&DSO_Setting_EE.HPos, DSO_Setting.HPos);
 	EE_Write((U8*)&DSO_Setting_EE.VPos, DSO_Setting.VPos);
 	EE_Write((U8*)&DSO_Setting_EE.TrigMode, DSO_Setting.TrigMode);
 	EE_Write((U8*)&DSO_Setting_EE.TrigEdge, DSO_Setting.TrigEdge);
 	EE_Write((U8*)&DSO_Setting_EE.TrigLvl, DSO_Setting.TrigLvl);
 	EE_Write((U8*)&DSO_Setting_EE.TrigSrc, DSO_Setting.TrigSrc);
 	EE_Write((U8*)&DSO_Setting_EE.TrigLvl_e, DSO_Setting.TrigLvl_e);

	// Write signature
 	EE_Write((U8*)&Signature_EE1, EE_signature1);
 	EE_Write((U8*)&Signature_EE2, EE_signature2);
 	}

 DSO_Status = 0;

 Focus = FC_TBRate;
 PrevFocus = FC_TBRate;
 PrevVPos = DSO_Setting.VPos;
 PrevTLvl = DSO_Setting.TrigLvl;
 PrevHPos = DSO_Setting.HPos;
 DetectSwitch();
 
 // Configure Test Signal output pin
 if(DSO_Setting.TrigSrc == TS_TrigOut) {
 	// If TrigOutput is selected set the Test Signal pin as normal port operation mode
	BitClr8(TCCR0, (1<<COM00));
 	}
 else {
 	// Make the pin controlled by Timer0
	BitSet8(TCCR0, (1<<COM00));
 	}

 // Initialize T0 scalers for DoCapture generation
 T0Scaler0 = 50;
 T0Scaler1 = 2;

 SysMode = ScopeMode;
 
 CmdInit();
 
 ScopePanel();
}

void	ScopePanel(void)
{
 ClearDBuf(DispBuf);

 Grid();
 Put_9x7(
 	pgm_read_byte((U8 *)IndicatorLoc + FC_VPos*2),
 	pgm_read_byte((U8 *)IndicatorLoc + FC_VPos*2 + 1),
	UpDown);	// VPos adjustment symbol
 Put_9x7(
 	pgm_read_byte((U8 *)IndicatorLoc + FC_HPos*2),
 	pgm_read_byte((U8 *)IndicatorLoc + FC_HPos*2 + 1),
	LeftRight);	// HPos adjustment symbol
 Put_9x7(
 	pgm_read_byte((U8 *)IndicatorLoc + FC_Triglvl*2),
 	pgm_read_byte((U8 *)IndicatorLoc + FC_Triglvl*2 + 1),
	UpDown);	// Triglevel adjustment symbol
 
 VLine(24 + Xofs, 61, 3);
 VLine(75 + Xofs, 61, 3);

 UpdateVSen();
 UpdateTrigMode();
 UpdateTimeBase();
 UpdateTrigEdge();
 UpdateTrigSrc();
 UpdateVPos();
 UpdateTLvl();
 UpdateHPos();
 UpdateHold();
 UpdateCpl();
 
// UpdateFocus();
 Hightlight(Focus);

 Display(DispBuf);
}


// Clear wave buffer
void	wClr(void) 
{
 U8	*ptmp;
 U16	tmp1;

 ptmp = WaveBuf;
 tmp1 = WLen * (Vmax/8);
 while(tmp1) {
	*ptmp++ = 0x00;
	tmp1--;
 	} 
}

// Draw a vertical line at (x, y) of length "len"
void	wVLine(U8 x, U8 y, U8 len)
{
 U8 *ptmp; 
 U8 tmp1, tmp2;
 
 if(len == 0) return;

 ptmp = WaveBuf + (U16)(((y & 0x3F) >> 3) * WLen)+ x;

 // -- Find the end point of the line
 len = y + len - 1; 
 if(len >= Vmax) {
 	len = Vmax - 1; 
	}

 // -- Find the number of bytes that include the line
 tmp2 = (len >> 3) - (y >> 3);
 switch(tmp2)
 	{ case 0: // Line start and end in the same byte
 		tmp1 = pgm_read_byte((U8 *)Bitmap1 + (y & 0x07));
		tmp2 = pgm_read_byte((U8 *)Bitmap2 + (len & 0x07));
		// -- Combine the two bitmap and write to display buffer
		tmp1 &= tmp2;
  		*ptmp |= tmp1;
		break;

   	  case 1: // Line falls in two bytes
 		// -- Create bitmap with 1s starts at the line's starting point   
 		tmp1 = pgm_read_byte((U8 *)Bitmap1 + (y & 0x07));
  		*ptmp |= tmp1;
		ptmp += WLen;
		tmp1 = pgm_read_byte((U8 *)Bitmap2 + (len & 0x07));
  		*ptmp |= tmp1;
		break;

	  default: // Line in multiple bytes
 		tmp1 = pgm_read_byte((U8 *)Bitmap1 + (y & 0x07));
  		*ptmp |= tmp1;
		ptmp += WLen;
	  
		// -- Write all middle bytes
	  	while(tmp2 > 1) {
  			*ptmp |= 0xFF;
			ptmp += WLen;
	  		tmp2--;
			}

 		// -- Create bitmap with 1s starts at the line's ending point
		tmp1 = pgm_read_byte((U8 *)Bitmap2 + (len & 0x07));
  		*ptmp |= tmp1;
		break;

	}
 
}

// Copy waveform data from Wavebuf to LCD
void	wDisplay(void)
{
 U8	*ptmp, *ptmp1;
 U8	tmp1, tmp2;

 ptmp = DispBuf + Xofs;
 ptmp1 = WaveBuf;

 LCD_Write(LCD_I_type, LCD_I_Page0, LCD_Block_Left);
 LCD_Write(LCD_I_type, LCD_I_Y0 + Xofs, LCD_Block_Left);
 LCD_Write(LCD_I_type, LCD_I_Page0, LCD_Block_Right);
 LCD_Write(LCD_I_type, LCD_I_Y0, LCD_Block_Right);

 // Transfer Block left & right data
 tmp1 = 0;
 while(tmp1 < 8) {
 	tmp2 = 0;
	while(tmp2 < (64 - Xofs)) {
		LCD_Write(LCD_D_type, *ptmp|*ptmp1, LCD_Block_Left);
		ptmp++;
		ptmp1++;
		tmp2++;
		}
 	tmp2 = 0;
	while(tmp2 < (WLen - 64 + Xofs)) {
		LCD_Write(LCD_D_type, *ptmp|*ptmp1, LCD_Block_Right);
		ptmp++;
		ptmp1++;
		tmp2++;
		}
	 // Increase X address
	tmp1++;

	LCD_Write(LCD_I_type, LCD_I_Page0|tmp1, LCD_Block_Left);
	LCD_Write(LCD_I_type, LCD_I_Page0|tmp1, LCD_Block_Right);
	LCD_Write(LCD_I_type, LCD_I_Y0 + Xofs, LCD_Block_Left);
	LCD_Write(LCD_I_type, LCD_I_Y0, LCD_Block_Right);
	ptmp += (Hmax - WLen);
	
 	}

}

void	AdjVPos(U8 Dir)
{
 if(Dir == UP) {
 	// -- Increase Ch A position
 	DSO_Setting.VPos += KeyParam;
 	if(DSO_Setting.VPos> Lmax) {
 		DSO_Setting.VPos = Lmax;
 		}
 	}
 else {
 	// -- Decrease Ch A position
 	if(DSO_Setting.VPos <= KeyParam) {
 		DSO_Setting.VPos= Lmin;
 		}
	else {
		DSO_Setting.VPos-= KeyParam;
		}
 	}

 EE_Write((U8*)&DSO_Setting_EE.VPos, DSO_Setting.VPos);

 UpdateVPos();
 //Display();
 PutBox(0, 0, 2, 61);
}

void	AdjTimeBase(U8 Dir)
{
 if(Dir == UP) {
 	// -- Increase TimeBase rate
 	if(++DSO_Setting.TimeBase == TB_max) {
 		DSO_Setting.TimeBase = TB_max - 1;
 		}
 	}
 else {
 	// -- Decrease TimeBase rate
 	if(--DSO_Setting.TimeBase == TB_min) {
 		DSO_Setting.TimeBase = TB_min + 1;
 		}	
 	}
 
 EE_Write((U8*)&DSO_Setting_EE.TimeBase, DSO_Setting.TimeBase);
 
 Hightlight(Focus);

 UpdateTimeBase();
 
 Hightlight(Focus);

 BitClr8(TIMSK , (1<<TICIE1));

 if(!BitTest8(DSO_Status, (1<<Hold))) {
 	DSO_Status = 0;
 	}
 
 //Display();
 PutBox(105, 23, 23, 9);
 }

void	AdjHPos(U8 Dir)
{
 if(Dir == UP) {
	DSO_Setting.HPos += KeyParam;
 	// -- Ensure HPosition not too close to the WB end
 	if(DSO_Setting.HPos >= (WBsize - WDsize)) {
 		DSO_Setting.HPos = (WBsize - WDsize);
 		}
 	}
 else {
 	// -- Ensure HPosition not too close to the WB start 
 	if(DSO_Setting.HPos <= KeyParam) {
 		DSO_Setting.HPos = 0;
 		}
 	else {
		DSO_Setting.HPos -= KeyParam;
 		}
 	}

 EE_Write((U8*)&DSO_Setting_EE.HPos, DSO_Setting.HPos);

 UpdateHPos();
 //Display();
 PutBox(24, 61, 52, 3);

 if(BitTest8(DSO_Status, (1<<Hold))) {
 	// Shift waveform. 
	ShowWave();
 	wDisplay();
 	}
 
}

void	AdjTrigMode(U8 Dir)
{
 if(Dir == UP) {
 	if(++DSO_Setting.TrigMode >= TM_Last) {
		DSO_Setting.TrigMode = TM_Auto;
		}
 	}
 else {
 	if(--DSO_Setting.TrigMode == TM_First) {
		DSO_Setting.TrigMode = TM_Single;
 		}
 	}

 EE_Write((U8*)&DSO_Setting_EE.TrigMode, DSO_Setting.TrigMode);

 Hightlight(Focus);

 UpdateTrigMode();

 Hightlight(Focus);

 //Update symbol display
 PutBox(105, 43, 23, 9);
}

void	AdjTrigEdge(U8 Dir)
{
 Dir = Dir;

 DSO_Setting.TrigEdge ^= 0x01;

 EE_Write((U8*)&DSO_Setting_EE.TrigEdge, DSO_Setting.TrigEdge);

 Hightlight(Focus);

 UpdateTrigEdge();
 
 Hightlight(Focus);
 
 //Display();
 PutBox(105, 52, 11, 9);
 
}

void	AdjTrigLvl(U8 Dir)
{ 
 U8 tmp0;
 
 if(DSO_Setting.TrigSrc == TS_External) {
 	tmp0 = DSO_Setting.TrigLvl_e;
 	}
 else {
 	tmp0 = DSO_Setting.TrigLvl;
 	}
 
 if(Dir == UP) {
 	// -- Increase trig level
 	tmp0+= KeyParam;
 	if(tmp0 > Lmax) {
 		tmp0 = Lmax;
 		}
 	}
 else {
 	if(tmp0 < KeyParam) {
 		tmp0 = Lmin - 1;
 		}
	else {
 		tmp0  -= KeyParam;
 		}
 	}

 if(DSO_Setting.TrigSrc == TS_External) {
 	DSO_Setting.TrigLvl_e = tmp0;
 	EE_Write((U8*)&DSO_Setting_EE.TrigLvl_e, DSO_Setting.TrigLvl_e);
 	}
 else {
 	DSO_Setting.TrigLvl = tmp0;
 	EE_Write((U8*)&DSO_Setting_EE.TrigLvl, DSO_Setting.TrigLvl);
 	}

 UpdateTLvl();
}

void	UpdateVSen(void)
{
 Puts1_ROM(VSenx, VSeny, (U8 *)(VSString + DSO_Setting.Vsen), DispBuf);

 PutBox(106, 15, 21, 21);
 }

void	UpdateTimeBase(void)
{
 Puts1_ROM(
 	pgm_read_byte((U8 *)IndicatorLoc + FC_TBRate*2),
 	pgm_read_byte((U8 *)IndicatorLoc + FC_TBRate*2 + 1),
 	(U8 *)(TBString + DSO_Setting.TimeBase),
 	DispBuf);

}

void	UpdateTrigMode(void)
{
 Puts1_ROM(
 	pgm_read_byte((U8 *)IndicatorLoc + FC_TrigMode*2),
 	pgm_read_byte((U8 *)IndicatorLoc + FC_TrigMode*2 + 1),
 	(U8 *)(TMString + DSO_Setting.TrigMode),
 	DispBuf);

}

void	UpdateTrigEdge(void)
{
 Put_9x7(
 	pgm_read_byte((U8 *)IndicatorLoc + FC_TrigEdge*2),
 	pgm_read_byte((U8 *)IndicatorLoc + FC_TrigEdge*2 + 1),
 	DSO_Setting.TrigEdge ? RisingEdge : FallingEdge);
 	
}

void	UpdateTrigSrc(void)
{
 U8 tmp;

 tmp = 'i';
 if(DSO_Setting.TrigSrc == TS_External) {
 	tmp = 'e';
 	}
  if(DSO_Setting.TrigSrc == TS_TrigOut) {
 	tmp = 'o';
 	}

 PutC5x7(
 	pgm_read_byte((U8 *)IndicatorLoc + FC_TrigMode*2) + 17,
 	pgm_read_byte((U8 *)IndicatorLoc + FC_TrigMode*2 + 1),
 	tmp,
 	DispBuf);	

 if(DSO_Setting.TrigSrc == TS_TrigOut) {
 	// If TrigOutput is selected set the Test Signal pin as normal port operation mode
	BitClr8(TCCR0, (1<<COM00));
 	}
 else {
 	// Make the pin controlled by Timer0
	BitSet8(TCCR0, (1<<COM00));
 	}

 // Test Signal pin is used as external trig input
 // Stop Test Signal output if external trig is selected and external input is connected 
 //	to the Test Signal pin. Otherwise restore Test Signal output.
 if(DSO_Setting.TrigSrc == TS_External) {
 	// Set the Test Signal output pin to input mode to stop its output
	BitClr8(DDRB, (1<<DDB4));
	BitSet8(PORTB, (1<<PB4));
 	}
 else {
 	// Set the Test Signal output pin to output mode to restore its output
	BitSet8(DDRB, (1<<DDB4));
 	}
 	 	
}

void	UpdateVPos(void)
{
 // Clear previous mark
 Triangle(MarkVPos, Lmax - PrevVPos, 1);
 // Put current mark
 Triangle(MarkVPos, Lmax - DSO_Setting.VPos, 0);

 PrevVPos= DSO_Setting.VPos;

 SetVPos();
 
}

void	UpdateTLvl(void)
{
 U8 tmp0;
 
 // Clear previous mark
 Triangle(MarkTLvl, Lmax - PrevTLvl, 1);
 // Put current mark
 if(DSO_Setting.TrigSrc == TS_External) {
 	tmp0 = DSO_Setting.TrigLvl_e;
 	}
 else {
 	tmp0 = DSO_Setting.TrigLvl;
 	}
 Triangle(MarkTLvl, Lmax - tmp0, 0);
 PutBox(103, 0, 2, 61);

 PrevTLvl = tmp0;

 SetTLvl();

}

void	UpdateHPos(void)
{
 // Clear previous mark
 rHLine(HPStart + (PrevHPos/5) + Xofs, 62, 20);

 
 // Put current mark
 HLine(HPStart + (DSO_Setting.HPos/5) + Xofs, 62, 20);

 PrevHPos = DSO_Setting.HPos;
 
}

void	UpdateHold(void)
{
 if(BitTest8(DSO_Status, (1<<Hold))) {
 	LcdPuts(HOLDx, HOLDy, (U8 *)"HOLD", DispBuf);
 	}
 else {
 	LcdPuts(HOLDx, HOLDy, (U8 *)"    ", DispBuf);
 	}
 
 PutBox(HOLDx, HOLDy, 23, 8);
 
}

void	UpdateCpl(void)
{

 switch(DSO_Setting.Cpl) {
 	case CP_AC: 
 		// First digit
 		PutChar5x5(CPLx, CPLy, 0);
 		// Second digit
 		PutChar5x5(CPLx + 6, CPLy, 1);
 		// Third digit
 		PutChar5x5(CPLx + 12, CPLy, 5);
		break;

 	case CP_DC: 
 		// First digit
 		PutChar5x5(CPLx, CPLy, 2);
 		// Second digit
 		PutChar5x5(CPLx + 6, CPLy, 1);
 		// Third digit
 		PutChar5x5(CPLx + 12, CPLy, 5);
		break;
		
 	case CP_GND: 
 		// First digit
 		PutChar5x5(CPLx, CPLy, 3);
 		// Second digit
 		PutChar5x5(CPLx + 6, CPLy, 4);
 		// Third digit
 		PutChar5x5(CPLx + 12, CPLy, 2);
		break;

	default:
		break;
 	}

 PutBox(106, 9, 17, 13);
 }

void	SetTLvl(void)
{
 U8 tmp;

 if(DSO_Setting.TrigSrc == TS_External) {
 	// -- External trig
 	tmp = DSO_Setting.TrigLvl_e << 2;
 	}
 else {
 	// -- Internal trig
 	tmp = DSO_Setting.TrigLvl;
 	tmp = (tmp<<1) + tmp;
 	tmp >>= 1;
 	tmp += 32;
 	}
 
 OCR3CH = 0x00;
 OCR3CL = tmp;
 
}

void	SetVPos(void)
{
 U8 tmp;
 
 tmp = DSO_Setting.VPos;
 tmp = (tmp<<1) + tmp;
 tmp >>= 1;
 tmp += 32;

 OCR3BH = 0x00;
 OCR3BL = tmp;

}

// =======================================================
//	Flash Display Mode - waveforms are displayed after a full buffer sampling had been
//		done.
//
void	FlashMode(void)
{
 if((BitTest8(DSO_Status, (1<<DoCapture))) 
 	&& !(BitTest8(DSO_Status, (1<<SampleDone)))
 	&& !(BitTest8(DSO_Status, (1<<Hold)))) {
 	
 	BitClr8(DSO_Status, (1<<DoCapture)|(1<<Abort));

	// -- Select and turn on trigger
	SetTrigger();
	// Stop Timer0 interrupt
	BitClr8(TIMSK, (1<<OCIE0));
	BitSet8(TIFR, (1<<OCF0));	// Clear pending interrupt

	CurrentSample = SampleBuf;
	// Dummy read count
	SampleCount = 0;
	BitSet8(DSO_Status, (1<<DummyRead));
	// Start Timer 1 and enable its interrupt
 	TCCR1A = (0<<COM1A1)|(0<<COM1A0) |(0<<COM1B1) |(0<<COM1B0) 
 			|(0<<COM1C1) |(0<<COM1C0) |(0<<WGM11) |(0<<WGM10);
	TCCR1B = (0<<ICNC1)|(0<<ICES1)|(1<<WGM13)|(1<<WGM12)
 			|(0<<CS12)|(0<<CS11)|(1<<CS10);
 	BitClr8(ACSR, (1<<ACIC));
	ICR1H = pgm_read_byte((U8*)(TBconstants + DSO_Setting.TimeBase) + 1);	
	ICR1L = pgm_read_byte((U8*)(TBconstants + DSO_Setting.TimeBase));	
	BitSet8(TIFR , (1<<ICF1));
	BitSet8(TIMSK , (1<<TICIE1));

	while(!BitTest8(DSO_Status, (1<<SampleDone))) {
		// Wait sampling being done in background driven by Timer 1 ISR
		if(BitTest8(DSO_Status, (1<<Abort))) {
			break;
			}
		}
	// Disable TOV3 interrupt
	BitClr8(TIMSK , (1<<TICIE1));

	BitSet8(DSO_Status, (1<<SampleDone));

	}
		
 if(BitTest8(DSO_Status, (1<<SampleDone))) {		
	// -- Copy newly captured wave date to wave display buffer

	if(!BitTest8(DSO_Status, (1<<Abort))) {
		// Reflash waveform display only at normal capture done
	
		// Locate display window
		if(BitTest8(DSO_Status, (1<<Trigged))) {
 			BitClr8(DSO_Status, (1<<Trigged));	

			// Keep the start address of current capture for waveform H shifting
			CaptureStart = (U8 *)(TrigAddress - TrigPos);
			if(CaptureStart < SampleBuf) {
				CaptureStart += WBsize;
				}
			// Enter HOLD state if in single mode
			if(DSO_Setting.TrigMode == TM_Single) {
				BitSet8(DSO_Status, (1<<SingleHold));
				}
			}
		
		else {
			// Keep the start address of current capture for waveform H shifting
			CaptureStart = SampleBuf;
			}
	
		ShowWave();
 		wDisplay();
		}
 
	// Start Timer0 interrupt
	T0Scaler0 = 50;
	T0Scaler1 = 2;
	BitSet8(TIFR, (1<<OCF0));	// Clear pending interrupt	
	BitSet8(TIMSK, (1<<OCIE0));
	
 	BitClr8(DSO_Status, (1<<SampleDone));
	}

}

// ==========================================================
//	Rolling Mode - waveforms are displayed as sampling progresses
//
void	RollingMode(void)
{
 if(BitTest8(DSO_Status, (1<<Hold))) {
 	// Hold
 	return;
 	}
 
 if(!BitTest8(DSO_Status, (1<<Rolling))) {
 	// Start rolling
 	BitSet8(DSO_Status, (1<<Rolling));
	BitSet8(DSO_Status, (1<<DummyRead));
	SampleCount = 128;
	CurrentSample = SampleBuf;
	SamplePacer = pgm_read_word((U16*)(TBconstants + DSO_Setting.TimeBase));
	wUpdatePacer = 1;
	BitClr8(TIMSK , (1<<TICIE1));
	// -- Select and turn on trigger
	SetTrigger();
	BitSet8(ACSR, (1<<ACI));	// Clear ACI flag

 	}
 else {
 	// Rolling in progress
 	if(BitTest8(DSO_Status, (1 << DoCapture))) {
		BitClr8(DSO_Status, (1 << DoCapture));		
		// Get one sample
		*CurrentSample = TempSample;
		CurrentSample += 1;
		if((U16)CurrentSample >= ((U16)SampleBuf + WBsize)) {
			CurrentSample = SampleBuf;
			}
		
		// Do this in Norm or Sing mode
		if(DSO_Setting.TrigMode != TM_Auto) {	
			// Do dummy reads if not finish yet
			if(BitTest8(DSO_Status, (1<<DummyRead))) {
				if(--SampleCount == 0) {
					BitClr8(DSO_Status, (1<<DummyRead));
					BitSet8(ACSR, (1<<ACI));	// Clear ACI flag
					}
				return;
				}
			
			// Check for trig if not happen yet
			if(!BitTest8(DSO_Status, (1<<Trigged))) {
				// No trig detected yet
				if(BitTest8(ACSR, (1<<ACI))) {
					// Trigged
					TrigAddress = (U16)CurrentSample;
					BitSet8(DSO_Status, (1<<Trigged));
					SampleCount = WBsize - TrigPos;
					// Set TrigOut to LOW
					BitClr8(Trigout_Port, (1<<Trigout_Bit));
					}
				}
			else {
				// Trigged. Count for the end of sampling
				if(--SampleCount == 0) {
					// Keep the start address of current capture for waveform H shifting
					CaptureStart = (U8 *)(TrigAddress - TrigPos);
					if(CaptureStart < SampleBuf) {
						CaptureStart += WBsize;
						}
					ShowWave();
 					wDisplay();

					// Prepare for next round of sampling
					DSO_Status = 0;

					// Enter HOLD state if in single mode
					if(DSO_Setting.TrigMode == TM_Single) {
						BitSet8(DSO_Status, (1<<SingleHold));
						}

					}
				}

			return;
			}

		// Update waveform in Auto mode
		if(--wUpdatePacer == 0) {
			wUpdatePacer = pgm_read_byte(
				(U8*)(UpdateConstants + DSO_Setting.TimeBase));
			
			// Keep the start address of current capture for waveform H shifting
			CaptureStart = (U8 *)(CurrentSample + 1);
			if(CaptureStart < SampleBuf) {
				CaptureStart += WBsize;
				}
			if(CaptureStart > (SampleBuf + WBsize)) {
				CaptureStart = (U8*)((U16)CaptureStart - WBsize);
				}
			
			// Keep TrigOut to HIGH in Auto mode
			BitSet8(Trigout_Port, (1<<Trigout_Bit));
			
			ShowWave();	
	 		wDisplay();		
			}
		
 		}

 	}

}
 
void	SetTrigger(void)
{
 // Select trig source
 if(DSO_Setting.TrigSrc == TS_External) {
 	// External trig
 	ADMUX = (0<<REFS1)|(0<<REFS0)|(0<<ADLAR)
 			|(0<<MUX4)|(0<<MUX3)|(0<<MUX2)|(1<<MUX1)|(1<<MUX0); 
 	}
 else {
 	// Internal trig
 	ADMUX = (0<<REFS1)|(0<<REFS0)|(0<<ADLAR)
 			|(0<<MUX4)|(0<<MUX3)|(1<<MUX2)|(0<<MUX1)|(0<<MUX0); 
 	}
 

 // Select effective trig edge
 BitClr8(ACSR, (1<<ACIS1)|(1<<ACIS0));
 if(DSO_Setting.TrigEdge == TE_Falling) {
 	// Note that signal is applied to IN- input of comparator
 	BitSet8(ACSR, (1<<ACIS1)|(1<<ACIS0));
 	}
 else {
 	BitSet8(ACSR, (1<<ACIS1));
 	}

 // Set TrigOut to HIGH
 BitSet8(Trigout_Port, (1<<Trigout_Bit));
 
}



