//////////////////////////////////////////////////////////////////////////////
//
// 	Filename:	DSO.h
//	Version:		
//
//	Author:		Liu, Zemin  (jyetek@gmail.com)
//	Company:	JYE Tech
//	Web: 		www.jyetech.com
//
//-----------------------------------------------------------------------------
//
// 	Target: 		ATmega64
// 	Tool chain: 	WinAvr (GCC-AVR)
//
//
//-----------------------------------------------------------------------------
//	Required files:
//
//-----------------------------------------------------------------------------
//	Notes:
//
//
//-----------------------------------------------------------------------------
//	Revision History:
//
///////////////////////////////////////////////////////////////////////////////

#ifndef	DSO_h

#define	DSO_h

#include	"Common.h"

// =======================================================
// Display parameters
#define	Lmax		62	// Maximun diaplayable wave level
#define	Lmin		2	// Minimun diaplayable wave level
#define	WBsize		256	// Wave buffer size
#define	WDsize		100	// Wave display size
#define	HPStart		24	// HPos indicator start position
#define	HPEnd		75	// HPos indicator end position

// Special indicator locations
#define	HOLDx	105			// "HOLD" location
#define	HOLDy	0

#define	CPLx	106
#define	CPLy	9

#define	VSenx	106
#define	VSeny	15

// =======================================================
//	Scope control type definitions
// =======================================================
//

// -- Time base type
typedef	enum	 {
	TB_min,
	TB_10m,	//		60s
	TB_5m,		//		30s
	TB_2m,		//		12s
	TB_1m,		//		6s
	TB_50s,		//		5s
	TB_20s,		// 		2s
	TB_10s,		// 1Hz	1s
	TB_5s,		// 2Hz	500ms
	TB_2s,		// 5Hz	200ms
	TB_1s,		// 10Hz	100ms
	TB_05s,		// 20Hz	50ms
	TB_02s,		// 50Hz	20ms
	TB_01s,		// 100Hz	10ms
	TB_50ms,	// 200Hz	5ms
	TB_20ms,	// 500Hz	2ms
	TB_10ms,	// 1K	1ms
	TB_5ms,		// 2K	500us
	TB_2ms,		// 5K	200us
	TB_1ms,		// 10K	100us
	TB_05ms,	// 20K	50us
	TB_02ms,	// 50K	20us (400 cycles)
	TB_01ms,	// 100K	10us (200 cycles)
	TB_max
	} TY_TimeBase;

// -- Trig Mode type
typedef	enum 	{
	TM_First,
	TM_Auto,
	TM_Normal,
	TM_Single,
	TM_Last
	}TY_TrigMode ;


// -- Trig Edge type
typedef 	enum 	 {
	TE_Falling,
	TE_Rising,
	}TY_TrigEdge;

typedef 	enum 	 {
	TS_Internal,
	TS_External,
	TS_TrigOut		// take this place to hold trig output setting
	}TY_TrigSrc;

typedef	struct 	{
	U8	TimeBase;
	U8	HPos;
	U8	VPos;
	U8	TrigMode;
	U8	TrigEdge;
	U8	TrigLvl;
	U8	TrigSrc;
	U8	TrigLvl_e;
	U8	Cpl;
	U8	Vsen;
	} DSO;

// -- V sensitivity type
typedef	enum 	{
	VS_First,
	VS_01V,
	VS_02V,
	VS_05V,
	VS_1V,
	VS_2V,
	VS_5V,
	VS_Last
	}TY_VSen ;

// -- Coupling type
typedef	enum 	{
	CP_DC,
	CP_AC,
	CP_GND
	}TY_Cpl ;

// -- Panel layout parameter structures
//  Highlight bar location and size (Y size is fixed to 9)
typedef	struct	{
	U8	x;
	U8	y;
	U8	xsize;	// 
	} FocusBox;

typedef	enum	{
	ScopeMode,
	FMeterMode
	} SystemMode;

// -- Adjustment direction
#define	UP			1
#define	DOWN		0

extern	FocusBox FBox[];
extern	U8 IndicatorLoc[];
extern	U8	SampleBuf[];
extern	U8	*CaptureStart;
extern	DSO		DSO_Setting;
extern	volatile	U8 DSO_Status;
extern	DSO		DSO_Setting_EE;
extern	volatile	U16	TrigPos;	
extern	volatile	U16	TrigPosC;	
extern	volatile	U8	SampleCount;
extern	volatile	U8	TempSample;
extern	U8	WaveBuf[];	
extern	volatile	U8	VSen;
extern	volatile	U8	Cpl;
extern	volatile	U16	WFmem;
extern	U8	WFmemIndex;
extern	U8 	*CurrentSample;
extern	volatile	U8	PrevTLvl;
extern	volatile	U16	SamplePacer;

extern	SystemMode SysMode;	

// DSO_Status bit allocation
#define	DoCapture		7
#define	SampleDone		6
#define	Trigged			5
#define	Abort			4
#define	Hold			3
#define	DummyRead		2
#define	Rolling			1
#define	SingleHold		0		// Set this bit when a capture in SINGLE mode is done

// Scope constants
#define	WFmemSize		288
#define	MaxWFmem		6		// Maximum number of waveform memories 

#define	EE_signature1	0xFA
#define	EE_signature2	0x5F

// -- Function Prototypes --

void	ScopeInit(void);
void	ScopePanel(void);

void	ShowWave(void);

void	ParamAdj(U8 Dir);
void	AdjVPos(U8 Dir);
void	AdjTimeBase(U8 Dir);
void	AdjHPos(U8 Dir);
void	AdjTrigMode(U8 Dir);
void	AdjTrigEdge(U8 Dir);
void	AdjTrigLvl(U8 Dir);

void	FlashMode(void);
void	RollingMode(void);
void	SetTrigger(void);

void	wClr(void);
void	wVLine(U8 x, U8 y, U8 len);
void	wDisplay(void);
void	UpdateVSen(void);
void	UpdateTimeBase(void);
void UpdateTrigMode(void);
void	UpdateTrigEdge(void);
void	UpdateTrigSrc(void);
void	UpdateVPos(void);
void	UpdateTLvl(void);
void	UpdateHPos(void);
void	UpdateHold(void);
void	UpdateCpl(void);
void	SetTLvl(void);
void	SetVPos(void);

#endif

