//////////////////////////////////////////////////////////////////////////////
//
// 	Filename:	FMeter.c
//	Version:		
//
//	Author:		Liu, Zemin  (jyetek@gmail.com)
//	Company:	JYE Tech
//	Web: 		www.jyetech.com
//
//-----------------------------------------------------------------------------
//
// 	Target: 		ATmega64
// 	Tool chain: 	WinAvr (GCC-AVR)
//
//
//-----------------------------------------------------------------------------
//	Required files:
//
//-----------------------------------------------------------------------------
//	Notes:
//
//-----------------------------------------------------------------------------
//	Revision History:
//
///////////////////////////////////////////////////////////////////////////////
//
//-----------------------------------------------------------------------------
// 	Includes
//-----------------------------------------------------------------------------

#include	"Common.h"
#include 	"FMeter.h"
#include	"Screen.h"
#include	"Board.h"
#include	"Command.h"

// ===========================================================
// 	File Scope Global variables
// ===========================================================
//
U8	MajorBuf[11] = {
	4,		// Big 'F'
	6,		// Big '='
	0,
	1,
	2,
	3,
	4,
	5,
	6,
	2,		// Big 'H'
	3		// Big 'z'
	};

U32	MResult;
U32	FMCapCurr;
U32	FMCapPrev;
U16	T3Ext;
U16	GateTiming;
U8	str[12];


// ===========================================================
// 	Function Definitions
// ===========================================================
//

void	FMeterInit(void)
{
 ClearDBuf(DispBuf);

 FMupdate();

 Display(DispBuf);

 // -------------------------------------------------------------------------
 //	Timer/Counter 1
 //
 
 // TCCRnA - Timer/Counter1(3) Control Register A
 // COMnA[1:0]: 	Compare Output Mode for Channel A
 // COMnB[1:0]: 	Compare Output Mode for Channel B
 // COMnC[1:0]: 	Compare Output Mode for Channel C
 //	Non-PWM Mode:
 //		00 - Normal port operation, OCnA/OCnB/OCnC disconnected.
 //		01 - Toggle OCnA/OCnB/OCnC on Compare Match.
 //		10 - Clear OCnA/OCnB/OCnC on Compare Match (Set output to low level).
 //		11 - Set OCnA/OCnB/OCnC on Compare Match (Set output to high level).
 //
 //	Fast PWM Mode:
 //		00 - Normal port operation, OCnA/OCnB/OCnC disconnected.
 //		01 - WGMn3:0 = 15: Toggle OCnA on Compare Match, OCnB/OCnC disconnected 
 //			(normal port operation). For all other WGMn settings, normal port operation,
//			OCnA/OCnB/OCnC disconnected.
 //		10 - Clear OCnA/OCnB/OCnC on Compare Match, set OCnA/OCnB/OCnC at TOP.
 //		11 - Set OCnA/OCnB/OCnC on Compare Match, clear OCnA/OCnB/OCnC at TOP.
 //
 //	Phase Correct and Phase and Frequency Correct PWM Mode:
 //		00 - Normal port operation, OCnA/OCnB/OCnC disconnected.
 //		01 - WGMn3:0 = 9 or 11: Toggle OCnA on Compare Match, OCnB/OCnC 
 //			disconnected (normal port operation). Forr all other WGMn settings, normal 
 //			port operation, OCnA/OCnB/OCnC disconnected.
 //		10 - Clear OCnA/OCnB/OCnC on Compare Match when upcounting. 
 //			Set OCnA/OCnB/OCnC on Compare Match when downcounting.
 //		11 - Set OCnA/OCnB/OCnC on Compare Match when up-counting.
 //			Clear OCnA/OCnB/OCnC on Compare Match when downcounting.
 //
 // WGMn[1:0]: 	Waveform Generation Mode. Combined with WGMn[3:2]  in the TCCRnB
 //				Mode			TOP			Update of OCRnX		TOVn Set
 //                      --------------------------------------------------------------
 //		0000 - 	Normal 			0xFFFF 		Immediate 			MAX
 //		0001 - 	PWM, PC8 		0x00FF 		TOP 					BOTTOM
 //		0010 - 	PWM, PC9 		0x01FF 		TOP 					BOTTOM
 //		0011 - 	PWM, PC10		0x03FF 		TOP 					BOTTOM
 //		0100 - 	CTC 			OCRnA 		Immediate 			MAX
 //		0101 - 	Fast PWM, 8-bit 	0x00FF 		TOP 					TOP
 //		0110 - 	Fast PWM, 9-bit 	0x01FF 		TOP 					TOP
 //		0111 - 	Fast PWM, 10-bit 	0x03FF 		TOP 					TOP
 //		1000 - 	PWM, P&F C		ICRn 		BOTTOM 				BOTTOM
 //		1001 - 	PWM, P&F C		OCRnA 		BOTTOM 				BOTTOM
 //		1010 - 	PWM, PC			ICRn		TOP 					BOTTOM
 //		1011 - 	PWM, PC			OCRnA 		TOP 					BOTTOM
 //		1100 - 	CTC 			ICRn 		Immediate 			MAX
 //		1101 - 	(Reserved) 		C 			C 					C
 //		1110 - 	Fast PWM 		ICRn 		TOP 					TOP
 //		1111 - 	Fast PWM 		OCRnA 		TOP 					TOP
 //
 // Current Settings:
 //	Normal mode
 TCCR1A = (0<<COM1A1)|(0<<COM1A0) |(0<<COM1B1) |(0<<COM1B0) 
 		|(0<<COM1C1) |(0<<COM1C0) |(0<<WGM11) |(0<<WGM10);
 
 // TCCRnB - Timer/Counter1(3) Control Register B
 //	ICNCn: 		Input Capture Noise Canceler
 //	ICESn: 		Input Capture Edge Select
 //	WGMn[3:2]: 	Waveform Generation Mode
 //	CSn2:0: 		Clock Select
 //		000 - No clock source (Timer/counter stopped).
 //		001 - clkI/O/1 (No prescaling)
 //		010 - clkI/O/8 (From prescaler)
 //		011 - clkI/O/64 (From prescaler)
 //		100 - clkI/O/256 (From prescaler)
 //		101 - clkI/O/1024 (From prescaler)
 //		110 - External clock source on Tn pin. Clock on falling edge.
 //		111 - External clock source on Tn pin. Clock on rising edge.
 //
 // Current Settings:
 // -- External clock on falling edge
 TCCR1B = (0<<ICNC1)|(0<<ICES1)|(0<<WGM13)|(0<<WGM12)
 		|(1<<CS12)|(1<<CS11)|(0<<CS10);

  // ----------------------------------------------------------------------
 //	Timer/Counter 2
 //
 // TCCR2 - Timer/Counter Control Register
 //	FOC2: 		Force Output Compare
 //	WGM2[1:0]: 	Waveform Generation Mode
 //			Mode		TOP			Update of OCR2	TOV2 Set
 //		00 - Normal		0xFF 		Immediate 		MAX
 //		01 - PWM, PC		0xFF 		TOP 				BOTTOM
 //		10 - CTC 		OCR2 		Immediate 		MAX
 //		11 - Fast PWM 	0xFF 		TOP 				MAX
 //
 //	COM2[1:0]: 	Compare Match Output Mode
 //		Non-PWM Mode:
 //			00 - Normal port operation, OC2 disconnected.
 //			01 - Toggle OC2 on Compare Match.
 //			10 - Clear OC2 on Compare Match.
 //			11 - Set OC2 on Compare Match.
 //
 //		Fast PWM Mode:
 //			00 - Normal port operation, OC2 disconnected.
 //			01 - Reserved
 //			10 - Clear OC2 on Compare Match, set OC2 at TOP.
 //			11 - Set OC2 on Compare Match, clear OC2 at TOP.
 //
 //		Phase Correct PWM Mode:
 //			00 - Normal port operation, OC2 disconnected.
 //			01 - Reserved
 //			10 - Clear OC2 on Compare Match when up-counting. Set OC2 on Compare
 //				Match when downcounting.
 //			11 - Set OC2 on Compare Match when up-counting. Clear OC2 on Compare
 //				Match when downcounting.
 //	CS22:0: 		Clock Select
 //			000 - No clock source (Timer/counter stopped).
 //			001 - clkI/O/(No prescaling)
 //			010 - clkI/O/8 (From prescaler)
 //			011 - clkI/O/64 (From prescaler)
 //			100 - clkI/O/256 (From prescaler)
 //			101 - clkI/O/1024 (From prescaler)
 //			110 - External clock source on T2 pin. Clock on falling edge.
 //			111 - External clock source on T2 pin. Clock on rising edge.
 //
 // Current settings:
 //	CTC mode (WGM2[1:0] = 10)
 //	1/256 prescaling (CS2[2:0] = 100)
 TCCR2 = (0<<FOC2)|(0<<WGM20)|(0<<COM21)|(0<<COM20)
 		|(1<<WGM21)|(1<<CS22)|(0<<CS21)|(0<<CS20);

 // TCNT2 - Timer/Counter Register
 //	
 TCNT2 = 0x00;

 // OCR2 - Output Compare Register
 //
 OCR2 = 124;	// 125*256/20 = 1.600ms


 // ----------------------------------------------------------------------
 //	Timer/Counter 3 (See descriptions in Timer/Counter 1 for meanings of settings)
 //
  // TCCRnA - Timer/Counter1(3) Control Register A
 // COMnA[1:0]: 	Compare Output Mode for Channel A
 // COMnB[1:0]: 	Compare Output Mode for Channel B
 // COMnC[1:0]: 	Compare Output Mode for Channel C
 //	Non-PWM Mode:
 //		00 - Normal port operation, OCnA/OCnB/OCnC disconnected.
 //		01 - Toggle OCnA/OCnB/OCnC on Compare Match.
 //		10 - Clear OCnA/OCnB/OCnC on Compare Match (Set output to low level).
 //		11 - Set OCnA/OCnB/OCnC on Compare Match (Set output to high level).
 //
 //	Fast PWM Mode:
 //		00 - Normal port operation, OCnA/OCnB/OCnC disconnected.
 //		01 - WGMn3:0 = 15: Toggle OCnA on Compare Match, OCnB/OCnC disconnected 
 //			(normal port operation). For all other WGMn settings, normal port operation,
//			OCnA/OCnB/OCnC disconnected.
 //		10 - Clear OCnA/OCnB/OCnC on Compare Match, set OCnA/OCnB/OCnC at TOP.
 //		11 - Set OCnA/OCnB/OCnC on Compare Match, clear OCnA/OCnB/OCnC at TOP.
 //
 //	Phase Correct and Phase and Frequency Correct PWM Mode:
 //		00 - Normal port operation, OCnA/OCnB/OCnC disconnected.
 //		01 - WGMn3:0 = 9 or 11: Toggle OCnA on Compare Match, OCnB/OCnC 
 //			disconnected (normal port operation). Forr all other WGMn settings, normal 
 //			port operation, OCnA/OCnB/OCnC disconnected.
 //		10 - Clear OCnA/OCnB/OCnC on Compare Match when upcounting. 
 //			Set OCnA/OCnB/OCnC on Compare Match when downcounting.
 //		11 - Set OCnA/OCnB/OCnC on Compare Match when up-counting.
 //			Clear OCnA/OCnB/OCnC on Compare Match when downcounting.
 //
 // WGMn[1:0]: 	Waveform Generation Mode. Combined with WGMn[3:2]  in the TCCRnB
 //				Mode			TOP			Update of OCRnX		TOVn Set
 //                      --------------------------------------------------------------
 //		0000 - 	Normal 			0xFFFF 		Immediate 			MAX
 //		0001 - 	PWM, PC8 		0x00FF 		TOP 					BOTTOM
 //		0010 - 	PWM, PC9 		0x01FF 		TOP 					BOTTOM
 //		0011 - 	PWM, PC10		0x03FF 		TOP 					BOTTOM
 //		0100 - 	CTC 			OCRnA 		Immediate 			MAX
 //		0101 - 	Fast PWM, 8-bit 	0x00FF 		TOP 					TOP
 //		0110 - 	Fast PWM, 9-bit 	0x01FF 		TOP 					TOP
 //		0111 - 	Fast PWM, 10-bit 	0x03FF 		TOP 					TOP
 //		1000 - 	PWM, P&F C		ICRn 		BOTTOM 				BOTTOM
 //		1001 - 	PWM, P&F C		OCRnA 		BOTTOM 				BOTTOM
 //		1010 - 	PWM, PC			ICRn		TOP 					BOTTOM
 //		1011 - 	PWM, PC			OCRnA 		TOP 					BOTTOM
 //		1100 - 	CTC 			ICRn 		Immediate 			MAX
 //		1101 - 	(Reserved) 		C 			C 					C
 //		1110 - 	Fast PWM 		ICRn 		TOP 					TOP
 //		1111 - 	Fast PWM 		OCRnA 		TOP 					TOP
 //
 // Current setting:
 // Normal mode
 TCCR3A = (0<<COM3A1)|(0<<COM3A0) |(0<<COM3B1) |(0<<COM3B0) 
 		|(0<<COM3C1) |(0<<COM3C0) |(0<<WGM31) |(0<<WGM30);
 
 // TCCRnB - Timer/Counter1(3) Control Register B
 //	ICNCn: 		Input Capture Noise Canceler
 //	ICESn: 		Input Capture Edge Select. 0 = falling, 1 = rising.
 //	WGMn[3:2]: 	Waveform Generation Mode
 //	CSn2:0: 		Clock Select
 //		000 - No clock source (Timer/counter stopped).
 //		001 - clkI/O/1 (No prescaling)
 //		010 - clkI/O/8 (From prescaler)
 //		011 - clkI/O/64 (From prescaler)
 //		100 - clkI/O/256 (From prescaler)
 //		101 - clkI/O/1024 (From prescaler)
 //		110 - External clock source on Tn pin. Clock on falling edge.
 //		111 - External clock source on Tn pin. Clock on rising edge.
 //
 // Current Settings:
 //	No prescaling
 TCCR3B = (0<<ICNC3)|(0<<ICES3)|(0<<WGM33)|(0<<WGM32)
 		|(0<<CS32)|(0<<CS31)|(1<<CS30);

 // -------------------------------------------------------------------
 //	Interrupt settings for Timer 1/2/3
 //
 // TIMSK - Timer/Counter Interrupt Mask Register
 //	OCIE2: 	1 = Timer/Counter2 Output Compare Match Interrupt Enable [TIMSK.7]
 //	TOIE2: 	1 = Timer/Counter2 Overflow Interrupt Enable [TIMSK.6]
 //	TICIE1: 	1 = Timer/Counter1, Input Capture Interrupt Enable [TIMSK.5]
 //	OCIE1A: 	1 = Timer/Counter1, Output Compare A Match Interrupt Enable [TIMSK.4]
 //	OCIE1B: 	1 = Timer/Counter1, Output Compare B Match Interrupt Enable [TIMSK.3]
 //	TOIE1: 	1 = Timer/Counter1, Overflow Interrupt Enable [TIMSK.2]
 // 	OCIE0: 	1 = Timer/Counter0 Output Compare Match Interrupt Enable [TIMSK.1]
 //	TOIE0: 	1 = Timer/Counter0 Overflow Interrupt Enable [TIMSK.0]
 TIMSK = (0<<OCIE2)|(0<<TOIE2)|(0<<TICIE1)|(0<< OCIE1A)
 	      	|(0<<OCIE1B)|(0<< TOIE1)|(0<< OCIE0)|(0<< TOIE0);

 // ETIMSK - Extended Timer/Counter Interrupt Mask Register
 //	TICIE3: 	Timer/Counter3, Input Capture Interrupt Enable
 //	OCIE3A: 	Timer/Counter3, Output Compare A Match Interrupt Enable
 //	OCIE3B: 	Timer/Counter3, Output Compare B Match Interrupt Enable
 //	TOIE3: 	Timer/Counter3, Overflow Interrupt Enable
 //	OCIE3C: 	Timer/Counter3, Output Compare C Match Interrupt Enable
 //	OCIE1C: 	Timer/Counter1, Output Compare C Match Interrupt Enable
 //
 ETIMSK = (0<<TICIE3)|(0<<OCIE3A)|(0<<OCIE3B)|(0<<TOIE3)
 		|(0<<OCIE3C)|(0<<OCIE1C);

 // TIFR - Timer/Counter Interrupt Flag Register
 //	OCF2: 	Output Compare Flag 2
 //	TOV2: 	Timer/Counter2 Overflow Flag
 //	ICF1: 	Timer/Counter1, Input Capture Flag
 //	OCF1A: 	Timer/Counter1, Output Compare A Match Flag
 //	OCF1B: 	Timer/Counter1, Output Compare B Match Flag
 //	TOV1: 	Timer/Counter1, Overflow Flag
 //	OCF0: 	Output Compare Flag 0
 //	TOV0: 	Timer/Counter0 Overflow Flag
 TIFR = (1<<OCF2)|(1<<TOV2)|(1<<ICF1)|(1<<OCF1A)
 		|(1<<OCF1B)|(1<<TOV1)|(1<<OCF0)|(1<<TOV0);

 // ETIFR - Extended Timer/Counter Interrupt Flag Register
 //	ICF3: 	Timer/Counter3, Input Capture Flag
 //	OCF3A: 	Timer/Counter3, Output Compare A Match Flag
 //	OCF3B: 	Timer/Counter3, Output Compare B Match Flag
 //	TOV3: 	Timer/Counter3, Overflow Flag
 //	OCF3C: 	Timer/Counter3, Output Compare C Match Flag
 //	OCF1C: 	Timer/Counter1, Output Compare C Match Flag
 //	
 ETIFR = (1<<ICF3)|(1<<OCF3A)|(1<<OCF3B)|(1<<TOV3)|(1<<OCF3C)|(1<<OCF1C);

}

// Update measurement results
void	FMupdate(void)
{
 //  
 PutCharNx16(Majorx, Majory, 10, MajorBuf[0]);			// Big 'F'
 PutCharNx16(Majorx + 11, Majory, 10, MajorBuf[1]);		// Big '='


 PutCharNx16(Majorx + 23, Majory, 7, MajorBuf[2]);		// MHz

 PutCharNx16(Majorx + 34, Majory, 7, MajorBuf[3]);		// 100KHz
 PutCharNx16(Majorx + 43, Majory, 7, MajorBuf[4]);		// 10KHz
 PutCharNx16(Majorx + 52, Majory, 7, MajorBuf[5]);		// 1KHz

 PutCharNx16(Majorx + 63, Majory, 7, MajorBuf[6]);		// 100Hz
 PutCharNx16(Majorx + 72, Majory, 7, MajorBuf[7]);		// 10Hz
 PutCharNx16(Majorx + 81, Majory, 7, MajorBuf[8]);		// 1Hz
 
 PutCharNx16(Majorx + 92, Majory, 10, MajorBuf[9]);		// Big 'H'
 PutCharNx16(Majorx + 103, Majory, 10, MajorBuf[10]);	// Big 'z'

 PutBox(Majorx, Majory, 110, 28);

}

void	MeasureFreq(void)
{
 U8 tmp0, tmp1;

 Put_9x7(GateSignx, GateSigny, (U8 *)GateSign);
 PutBox(GateSignx, GateSigny, 9, 8);
 
 MResult = 0;
 Gate1s();

 Put_9x7(GateSignx, GateSigny, (U8 *)Blank9X8);
 PutBox(GateSignx, GateSigny, 9, 8);
 
 BinToDec32(MResult, str);

 tmp1 = 0;
 tmp0 = 0;
 while(tmp0 < 7) {
 	MajorBuf[2 + tmp0] = str[3 + tmp0];
	if(tmp1 == 0) {
		// Blank out leading zeros when tmp1 = 0
		if(MajorBuf[2 + tmp0] == 0) {
			MajorBuf[2 + tmp0] = 11;	// set to blank
			}
		else {
			tmp1 = 1;
			}
		}
	tmp0++;
 	}

 FMupdate();
 Delay(60000);
}



