; //////////////////////////////////////////////////////////////////////////////
; //
; // 	Filename:	Zmon.S
; //	Version:		
; //
; //	Author:		Liu, Zemin   (jyetek@gmail.com)
; //	Company:	JYE Tech
; //	Web: 		www.jyetech.com
; //
; //-----------------------------------------------------------------------------
; //
; // 	Target: 		ATmega64
; // 	Tool chain: 	WinAvr (GCC-AVR)
; //
; //-----------------------------------------------------------------------------


#include <avr/io.h>
#include <avr/Sfr_defs.h>

#define	TM_AUTO		1
#define	TM_NORM	2
#define	TM_SING		3

#define	TB_20ms		15
#define	WBsize		256

#define	DoCapture		7
#define	SampleDone		6
#define	Trigged			5
#define	Abort			4
#define	Hold				3
#define	DummyRead		2

#define	ADC_IN_PORT	PINA
#define	ADC_CLK_PORT	PORTB
#define	ADC_CLK_BIT		PB7
#define	Trigout_Port		PORTB
#define	Trigout_Bit		PB4

#define	KeyScanWait		30		; Wait for 30ms if not key pressed

	.extern	SampleBuf
	.extern	CurrentSample
	.extern	SampleCount
	.extern	TrigAddress
	.extern	DSO_Status
	.extern	DSO_Setting
	.extern	TempSample
	.extern	SamplePacer
	.extern	DoScan
	.extern	ScanTimer
	.extern	TrigPos
	.extern	TrigPosC
	.extern	TBconstants
	
	.extern	MResult
	.extern	GateTiming
	.extern	FMCapCurr
	.extern	FMCapPrev
	.extern	T3Ext

	.extern	SysMode

	.extern	T0Scaler0
	.extern	T0Scaler1

	.section	.data
	
       .section 	.text
        
	.global	zmon   
zmon:
       
       ret                    		;back to pgm

 	.global	delay1us
delay1us: 	
 	nop
 	nop
 	nop
 	nop
 	nop
 	
 	nop
 	nop
 	nop
 	nop
 	nop
 	
 	nop
 	nop
 	nop
 	nop
 	nop
 	
 	nop
	ret 	

        .global 	SIG_OUTPUT_COMPARE0                
        ; OC0 interrupt happens at about 1ms regardless FlashMode or RollingMode.
        ;	At FlashMode it paces for the capture. At RollingMode it drives ADC 
        ;	and SamplePacer.
SIG_OUTPUT_COMPARE0:
	push		r16
	push		r24
	push		r25
	push		r26
	push		r27
	push		r30
	push		r31
	in		r16, _SFR_IO_ADDR(SREG)

	lds		r24, DSO_Setting+0		; Get TimeBase setting
	cpi		r24, TB_20ms
	brsh		OV0_5

	; Decrease SamplePacer in RollingMode
	lds		r24, SamplePacer
	lds		r25, SamplePacer+1
	sbiw		r24, 1
	; Check if SamplePacer is 0	
	movw	r26, r24
	or		r26, r25
	brne		OV0_3
	lds		r24, DSO_Status
	ori		r24, 0x80					; Set DoCapture to 1
	sts		DSO_Status, r24

	ldi		r30, lo8(TBconstants)
	ldi		r31, hi8(TBconstants)
	lds		r24, DSO_Setting+0
	ldi		r25, 0
	add		r24, r24
	adc		r25, r25
	add		r30, r24
	adc		r31, r25
	lpm		r24, Z+
	lpm		r25, Z
OV0_3:
	sts		SamplePacer, r24
	sts		SamplePacer+1, r25

OV0_4:	
	; Do background sampling in RollingMode
	cbi		_SFR_IO_ADDR(ADC_CLK_PORT), ADC_CLK_BIT		; 1
	in		r24, _SFR_IO_ADDR(ADC_IN_PORT)			; 1
	; Save sample
	sts		TempSample, r24
	sbi		_SFR_IO_ADDR(ADC_CLK_PORT), ADC_CLK_BIT		; 1
	rjmp		OV0_9

	; Timing for capture when TimeBase >= TB_20ms (FlashMode)
OV0_5:
	lds		r24, T0Scaler0
	dec		r24
	brne		OV0_8
	lds		r24, T0Scaler1
	dec		r24
	brne		OV0_7

	lds		r24, DSO_Status
	ori		r24, 0x80					; Set DoCapture to 1
	sts		DSO_Status, r24
	
	ldi		r24, 2
OV0_7:
	sts		T0Scaler1, r24
	ldi		r24, 50
OV0_8:	
	sts		T0Scaler0, r24
OV0_9:	
	; Signal for key scan if ScanTimer is not 0
	lds		r24, ScanTimer
	or		r24, r24
	breq		OV0_901
	ldi		r24, 1
	sts		DoScan, r24
	
OV0_901:	
	out		_SFR_IO_ADDR(SREG), r16
	pop		r31
	pop		r30
	pop		r27
	pop		r26
	pop		r25
	pop		r24
	pop		r16
	reti

	; Sampling ISR in FlashMode
	;
        .global 	SIG_INPUT_CAPTURE1              
SIG_INPUT_CAPTURE1:
	push		r16

	push		r26
	push		r27

	push		r24
	push		r25
	
	in		r16, _SFR_IO_ADDR(SREG)

	; FlashMode sampling when TimeBase > TB_50ms
ADC_0:										;
	cbi		_SFR_IO_ADDR(ADC_CLK_PORT), ADC_CLK_BIT		
	in		r24, _SFR_IO_ADDR(ADC_IN_PORT)					
	; Get current sample save address
	lds		r26, CurrentSample
	lds		r27, CurrentSample+1
	st		X, r24											
	inc		r26
	; Keep current sample save address
	sts		CurrentSample, r26
	sts		CurrentSample+1, r27
	sbi		_SFR_IO_ADDR(ADC_CLK_PORT), ADC_CLK_BIT		

	; Count down 
	lds		r24, SampleCount
	dec		r24
	sts		SampleCount, r24
	
	lds		r25, DSO_Status
	sbrs		r25, DummyRead
	rjmp		ADC_2
	; Dummy read in progress
	brne		ADC_8							; Check SampleCount
	; Dummy read done. Clear DummyRead
	andi		r25, 0xFB
	sts		DSO_Status, r25
	; Clear ACI flag in ACSR
	sbi		_SFR_IO_ADDR(ACSR), 4			
	rjmp		ADC_8
ADC_2:										; Normal capture in progress
	; Check if it is after trig
	sbrc		r25, Trigged
	rjmp		ADC_6
	; Check if trig happened
	sbic		_SFR_IO_ADDR(ACSR), 4			; 1/2/3	Check ACI flag
	rjmp		ADC_9
ADC_3:	
	brne		ADC_8							; The Z flag reflects r24, i.e. SampleCount
	; SampleCount =0, this means either 256 samples have been done without trigging,
	;	or sampling after trig has finished
	; Check trig mode
	lds		r24, DSO_Setting+3
	cpi		r24, TM_AUTO
	breq		ADC_7
	; Normal or Single mode. Continue if not abort
	sbrs		r25, Abort
	rjmp		ADC_8	
	rjmp		ADC_7

ADC_6:
	; Trig has happened. Check for end of sampling
	brne		ADC_8
ADC_7:	
	; Abort flag set. Flag SampleDone
	ori		r25, 0x40
	sts		DSO_Status, r25
ADC_8:	
	out		_SFR_IO_ADDR(SREG), r16

	pop		r25
	pop		r24
	
	pop		r27
	pop		r26
	
	pop		r16

	reti

ADC_9:										; Trig detected
	; Keep trig address
	sts		TrigAddress, r26
	sts		TrigAddress+1, r27
	; Set TrigOut to LOW
	cbi		_SFR_IO_ADDR(Trigout_Port), Trigout_Bit			; 2
	
	; Flag trigged
	ori		r25, 0x20					; Set Trigged to 1
	sts		DSO_Status, r25

	lds		r24, TrigPosC + 0
	sts		SampleCount, r24
	rjmp		ADC_8



	; Key press ISR
	;
        .global 	SIG_INTERRUPT0          
        .global 	SIG_INTERRUPT1          
        .global 	SIG_INTERRUPT2          
        .global 	SIG_INTERRUPT3          
SIG_INTERRUPT0:
SIG_INTERRUPT1:
SIG_INTERRUPT2:
SIG_INTERRUPT3:
	push		r16
	in		r16, _SFR_IO_ADDR(SREG)
	push		r24

	lds		r24, DSO_Status
	ori		r24, 0x10				; Set Abort bit
	sts		DSO_Status, r24

	ldi		r24, KeyScanWait
	sts		ScanTimer, r24			
	
	pop		r24
	out		_SFR_IO_ADDR(SREG), r16
	pop		r16

	reti


	; Timing for 1 second
	;
	.global	Gate1s
Gate1s:
	push		r24
	push		r25

	; Clear Timer 2 OCF2 flag and load it with initial value for 1s timing
	ldi		r24, hi8(625)
	sts		GateTiming+1, r24
	ldi		r24, lo8(625)
	sts		GateTiming+0, r24
	ldi		r24, 0
	out		_SFR_IO_ADDR(TCNT2), r24		; clear TCNT2
	ldi		r24, 0x80
	out		_SFR_IO_ADDR(TIFR), r24		; clear OCF2 flag
		
	; Clear Timer 1 and its overflow flag
	clr		r24
	out		_SFR_IO_ADDR(TCNT1H), r24
	out		_SFR_IO_ADDR(TCNT1L), r24
	ldi		r24, 0x04
	out		_SFR_IO_ADDR(TIFR), r24
	
	
	; Check for end of timing while looking after Timer 1 and Timer 2 overflow 
G1s100:
	; Check Timer 1 for overflow
	in 		r24, _SFR_IO_ADDR(TIFR)
	sbrc		r24, 2
	rjmp		G1s104
G1s101:	
	; Check Timer 2 ovreflow (OCF2 flag)
	in 		r24, _SFR_IO_ADDR(TIFR)
	sbrc		r24, 7
	rjmp		G1s105
G1s102:	
	; Check Timer 3 capture
	lds		r24, ETIFR
	sbrc		r24, 5
	rjmp		G1s106
G1s103:
	; Check for Timer 3 overflow
	lds		r24, ETIFR
	sbrs		r24, 2
	rjmp		G1s100
	
	; Timer 3 overflowed
	ldi		r24, 0x04
	sts		ETIFR, r24				// Clear flag

	lds		r24, T3Ext+0
	inc		r24
	sts		T3Ext+0, r24
	brne		G1s100
	lds		r24, T3Ext+1
	inc		r24
	sts		T3Ext+1, r24
	rjmp		G1s100
	
G1s104:
	; Timer 1 overflowed
	ldi		r24, 0x04
	out		_SFR_IO_ADDR(TIFR), r24	// Clear flag
	
	lds		r24, MResult+2
	inc		r24
	sts		MResult+2, r24
	rjmp		G1s101
	
G1s105:
	; Timer 2 overflowed
	ldi		r24, 0x80
	out		_SFR_IO_ADDR(TIFR), r24	// Clear flag

	lds		r24, GateTiming+0
	lds		r25, GateTiming+1
	sbiw		r24, 1
	sts		GateTiming+0, r24
	sts		GateTiming+1, r25
	or		r24, r25
	brne		G1s102
	rjmp		G1s200
	
G1s106:
	; T3 capture happened
	ldi		r24, 0x20
	sts		ETIFR, r24				// Clear flag
	
	lds		r24, FMCapCurr+0		; Save previous capture
	sts		FMCapPrev+0, r24
	lds		r24, FMCapCurr+1
	sts		FMCapPrev+1, r24
	lds		r24, FMCapCurr+2
	sts		FMCapPrev+2, r24
	lds		r24, FMCapCurr+3
	sts		FMCapPrev+3, r24
	
	lds		r24, ICR3L			; Keep current capture
	sts		FMCapCurr+0, r24
	lds		r24, ICR3H
	sts		FMCapCurr+1, r24
	lds		r24, T3Ext+0
	sts		FMCapCurr+2, r24
	lds		r24, T3Ext+1
	sts		FMCapCurr+3, r24
	
	rjmp		G1s103

	; Gate timeout
G1s200:
	; read Timer 1
	in		r24, _SFR_IO_ADDR(TCNT1L)
	sts		MResult+0, r24
	in		r24, _SFR_IO_ADDR(TCNT1H)
	sts		MResult+1, r24

	pop		r25
	pop		r24
	ret

.end




